Type: Package
Package: rfm
Title: Recency, Frequency and Monetary Value Analysis
Version: 0.4.0
Authors@R: person("Aravind", "Hebbali", email = "hebbali.aravind@gmail.com", role = c("aut", "cre"),
    comment = c(ORCID = "0000-0001-9220-9669"))
Description: Tools for RFM (recency, frequency and monetary value) analysis. 
    Generate RFM score from both transaction and customer level data. Visualize the
    relationship between recency, frequency and monetary value using heatmap, 
    histograms, bar charts and scatter plots. Includes a 'shiny' app for 
    interactive segmentation. References:
    i. Blattberg R.C., Kim BD., Neslin S.A (2008) <doi:10.1007/978-0-387-72579-6_12>.
License: MIT + file LICENSE
URL: https://github.com/rsquaredacademy/rfm,
        https://rfm.rsquaredacademy.com/
BugReports: https://github.com/rsquaredacademy/rfm/issues
Depends: R (>= 3.2)
Imports: dplyr, gganimate, ggplot2, magrittr, plotly, RColorBrewer,
        rlang, scales, stats, treemapify, utils, xplorerr
Suggests: cli, covr, DT, kableExtra, knitr, rmarkdown, rmdformats,
        testthat (>= 3.0.0), vdiffr
VignetteBuilder: knitr
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-04-30 07:56:04 UTC; HP
Author: Aravind Hebbali [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-9220-9669>)
Maintainer: Aravind Hebbali <hebbali.aravind@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-30 09:00:02 UTC
Built: R 4.6.0; ; 2026-05-01 22:15:42 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(print,rfm_table_customer)
S3method(print,rfm_table_order)
S3method(rfm_table_customer,default)
S3method(rfm_table_order,default)
export(rfm_bar_chart)
export(rfm_barchart_data)
export(rfm_create_report)
export(rfm_fm_plot)
export(rfm_heatmap)
export(rfm_heatmap_data)
export(rfm_histograms)
export(rfm_launch_app)
export(rfm_order_dist)
export(rfm_plot_bar_chart)
export(rfm_plot_heatmap)
export(rfm_plot_histogram)
export(rfm_plot_median_frequency)
export(rfm_plot_median_monetary)
export(rfm_plot_median_recency)
export(rfm_plot_order_dist)
export(rfm_plot_revenue_dist)
export(rfm_plot_segment)
export(rfm_plot_segment_scatter)
export(rfm_plot_segment_summary)
export(rfm_rf_plot)
export(rfm_rm_plot)
export(rfm_segment)
export(rfm_segment_summary)
export(rfm_table_customer)
export(rfm_table_customer_2)
export(rfm_table_order)
importFrom(RColorBrewer,brewer.pal)
importFrom(dplyr,arrange)
importFrom(dplyr,between)
importFrom(dplyr,distinct)
importFrom(dplyr,everything)
importFrom(dplyr,group_by)
importFrom(dplyr,join_by)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,pull)
importFrom(dplyr,rename)
importFrom(dplyr,select)
importFrom(dplyr,summarise)
importFrom(ggplot2,aes)
importFrom(ggplot2,coord_flip)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_line)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,facet_grid)
importFrom(ggplot2,geom_bar)
importFrom(ggplot2,geom_histogram)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,geom_tile)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,labs)
importFrom(ggplot2,position_dodge)
importFrom(ggplot2,scale_fill_gradientn)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,sec_axis)
importFrom(ggplot2,theme)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(ggplot2,ylim)
importFrom(magrittr,"%<>%")
importFrom(magrittr,"%>%")
importFrom(magrittr,add)
importFrom(magrittr,extract)
importFrom(magrittr,multiply_by)
importFrom(magrittr,set_names)
importFrom(magrittr,use_series)
importFrom(rlang,":=")
importFrom(rlang,.data)
importFrom(rlang,int)
importFrom(stats,median)
importFrom(stats,quantile)
importFrom(stats,reorder)
importFrom(stats,runif)
importFrom(treemapify,geom_treemap)
importFrom(treemapify,geom_treemap_text)
importFrom(utils,available.packages)
importFrom(utils,browseURL)
importFrom(utils,install.packages)
importFrom(utils,menu)
importFrom(utils,packageVersion)
importFrom(utils,update.packages)

<br>
<br>
<div align="left">
<img style="width:400px" src="https://github.com/sonwanesuresh95/rfm/blob/main/example_/rfm-logos_black.png"><br>
</div>
<br>

[![PyPI Latest Release](https://img.shields.io/pypi/v/rfm.svg)](https://pypi.org/project/rfm/)
![PyPI](https://badge.fury.io/py/rfm.svg)
[![Package Status](https://img.shields.io/pypi/status/rfm.svg)](https://pypi.org/project/rfm/)
[![License](https://img.shields.io/pypi/l/rfm.svg)](https://github.com/sonwanesuresh95/rfm/blob/main/LICENSE)
![Downloads Per Month](https://img.shields.io/pypi/dm/rfm)

# rfm
<b>rfm: Python Package for RFM Analysis and Customer Segmentation</b>

## Info

**rfm** is a Python package that provides **recency, frequency, monetary analysis** results
for a certain transactional dataset within a snap. Its flexible structure and multiple automated
functionalities provide easy and intuitive approach to RFM Analysis in an automated fashion.
It aims to be a ready-made python package with high-level and quick prototyping.
On practical hand, **real world data** is easily suited and adapted by the package.
Additionally, it can make colorful, intuitive graphs using a matplotlib backend without 
breaking a sweat.

## Installation
### Dependencies
<ul>
  <li>Python (>=3.7)</li>
  <li>Pandas (>=1.2.4)</li>
  <li>NumPy (>=1.20.1)</li>
  <li>matplotlib (>=3.3.4)</li>
</ul>

To install the current release (Ubuntu and Windows):

```
$ pip install rfm
```

## Usage

```
# predefine a transaction dataset as df

>>> from rfm import RFM

>>> r = RFM(df, customer_id='CustomerID', transaction_date='InvoiceDate', amount='Amount')

>>> r.plot_segment_distribution()
```

<div align="left">
  <img style="width:550px" src="https://github.com/sonwanesuresh95/rfm/blob/main/example_/rfm_seg_dist.png"><br>
</div>


# License
[MIT](https://github.com/sonwanesuresh95/rfm/blob/main/LICENSE)

# Documentation
<-- Temporarily Hosted Here -->
## Initialization
Read required dataframe
```
>>> df = pd.read_csv('~./data.csv')
```

Import RFM package and start rfm analysis automatically:
```
>>> from rfm import RFM

>>> r = RFM(df, customer_id='CustomerID', transaction_date='InvoiceDate', amount='Amount') 

>>> r.rfm_table
```
If you want to do rfm analysis manually:
```
>>> r = RFM(df, customer_id='CustomerID', transaction_date='InvoiceDate', amount='Amount', automated=False)
```

## Attributes

### RFM.rfm_table
returns resultant rfm table df generated with recency, frequency & monetary values and scores along with segments
```
>>> r.rfm_table
```
<div align="left">
  <img style="width:500px" src="https://github.com/sonwanesuresh95/rfm/blob/main/example_/rfm_table.png"><br>
</div>

### RFM.segment_table
returns segment table df with 10 unique categories i.e. Champions, Loyal Accounts etc. 
```
>>> r.segment_table
```
<div align="left">
  <img style="height:250px" src="https://github.com/sonwanesuresh95/rfm/blob/main/example_/rfm_segment_table.png"><br>
</div>

## Methods

### RFM.plot_rfm_histograms()
Plots recency, frequency and monetary histograms in a single row
```
>>> r.plot_rfm_histograms()
```
<div align="left">
  <img style="width:700px" src="https://github.com/sonwanesuresh95/rfm/blob/main/example_/rfm_histograms.png"><br>
</div>

### RFM.plot_rfm_order_distribution()
Plots orders by customer number
```
>>> r.plot_rfm_order_distribution()
```
<div align="left">
  <img style="width:700px" src="https://github.com/sonwanesuresh95/rfm/blob/main/example_/rfm_order_dist.png"><br>
</div>

### RFM.plot_versace_plot(column1, column2)
Plots scatterplot of two input columns

```
>>> r.plot_versace_plot(column1='recency',column2='monetary_value')
```

<div align="left">
  <img style="width:550px" src="https://github.com/sonwanesuresh95/rfm/blob/main/example_/rfm_rm.png"><br>
</div>

```
>>> r.plot_versace_plot(column1='recency',column2='frequency')
```

<div align="left">
  <img style="width:550px" src="https://github.com/sonwanesuresh95/rfm/blob/main/example_/rfm_rf.png"><br>
</div>

```
>>> r.plot_versace_plot(column1='frequency',column2='monetary_value')
```

<div align="left">
  <img style="width:550px" src="https://github.com/sonwanesuresh95/rfm/blob/main/example_/rfm_fm.png"><br>
</div>

### RFM.plot_distribution_by_segment(column, take)
Plots Distribution of input column by segment
```
>>> r.plot_distribution_by_segment(column='recency',take='median')
```
<div align="left">
  <img style="width:550px" src="https://github.com/sonwanesuresh95/rfm/blob/main/example_/rfm_mrdian_rec.png"><br>
</div>

```
>>> r.plot_distribution_by_segment(column='frequency',take='median')
```
<div align="left">
  <img style="width:550px" src="https://github.com/sonwanesuresh95/rfm/blob/main/example_/rfm_median_freq.png"><br>
</div>

```
>>> r.plot_distribution_by_segment(column='monetary_value',take='median')
```
<div align="left">
  <img style="width:550px" src="https://github.com/sonwanesuresh95/rfm/blob/main/example_/rfm_med_mon.png"><br>
</div>

### RFM.plot_column_distribution(column)
Plots column distribution of input column
```
>>> r.plot_column_distribution(column='recency')
```
<div align="left">
  <img style="width:550px" src="https://github.com/sonwanesuresh95/rfm/blob/main/example_/rfm_col_dist_rec.png"><br>
</div>

```
>>> r.plot_column_distribution(column='frequency')
```
<div align="left">
  <img style="width:550px" src="https://github.com/sonwanesuresh95/rfm/blob/main/example_/rfm_col_dist_freq.png"><br>
</div>

```
>>> r.plot_column_distribution(column='monetary_value')
```
<div align="left">
  <img style="width:550px" src="https://github.com/sonwanesuresh95/rfm/blob/main/example_/rfm_col_dist_mon.png"><br>
</div>

### RFM.plot_segment_distribution()
```
>>> r.plot_segment_distribution()
```
Plots Segment Distribution, i.e. Segments vs no. of customers
<div align="left">
  <img style="width:550px" src="https://github.com/sonwanesuresh95/rfm/blob/main/example_/rfm_seg_dist.png"><br>
</div>

### RFM.find_customers(segment)
returns rfm results df with input category
```
>>> r.find_customers('Champions')
```
<div align="left">
  <img style="width:550px" src="https://github.com/sonwanesuresh95/rfm/blob/main/example_/rfm_champions.png"><br>
</div>

## ----echo=FALSE, message=FALSE------------------------------------------------
library(rfm)
library(dplyr)
library(rlang)
library(scales)
library(knitr)
library(kableExtra)
library(magrittr)
library(ggplot2)
library(DT)
library(grDevices)
library(RColorBrewer)
options(knitr.table.format = "html")

## ----rfm_data_orders----------------------------------------------------------
head(rfm_data_orders)

## ----rfm_data_customer--------------------------------------------------------
head(rfm_data_customer)

## ----rfm_table_order, eval=FALSE----------------------------------------------
# analysis_date <- as.Date("2006-12-31")
# rfm_result <- rfm_table_order(rfm_data_orders, customer_id, order_date, revenue, analysis_date)
# rfm_result

## ----rfm_table_order2, eval=TRUE, echo=FALSE----------------------------------
analysis_date <- as.Date("2006-12-31")
rfm_result <- rfm_table_order(rfm_data_orders, customer_id, order_date, revenue, analysis_date)
rfm_result$rfm[1:10, ] %>%
  select(customer_id, recency_days, transaction_count, amount, rfm_score, recency_score, frequency_score, monetary_score, first_name, last_name, email) %>% 
  kable() %>%
  kable_styling()

## ----segments, echo=FALSE-----------------------------------------------------
segment <- c("Champions", "Potential Loyalist", "Loyal Customers", 
             "Promising", "New Customers", "Can't Lose Them", 
             "At Risk", "Need Attention", "About To Sleep", "Lost")
description <- c(
  "Bought recently, buy often and spend the most",
  "Recent customers, spent good amount, bought more than once",
  "Spend good money. Responsive to promotions",
  "Recent shoppers, but haven't spent much",
  "Bought more recently, but not often",
  "Made big purchases and often, but long time ago",
  "Spent big money, purchased often but long time ago",
  "Above average recency, frequency & monetary values",
  "Below average recency, frequency & monetary values",
  "Bought a long time ago, average amount spent"
)
recency <-   c("5", "3 - 5", "2 - 4", "3 - 4", "4 - 5", "1 - 2", "1 - 2", "1 - 3", "2 - 3", "1 - 1")
frequency <- c("5", "3 - 5", "2 - 4", "1 - 3", "1 - 3", "3 - 4", "2 - 5", "3 - 5", "1 - 3", "1 - 5")
monetary <-  c("5", "2 - 5", "2 - 4", "3 - 5", "1 - 5", "4 - 5", "4 - 5", "3 - 5", "1 - 4", "1 - 5")
segments <- data.frame(
  Segment = segment, Description = description,
  R = recency, `F` = frequency, M = monetary
)

segments %>%
  kable() %>%
  kable_styling(full_width = FALSE, font_size = 12)

## ----criteria, echo=FALSE-----------------------------------------------------
segment_names <- c("Champions", "Potential Loyalist", "Loyal Customers", 
                   "Promising", "New Customers", "Can't Lose Them", 
                   "At Risk", "Need Attention", "About To Sleep", "Lost")

recency_lower <-   c(5, 3, 2, 3, 4, 1, 1, 1, 2, 1)
recency_upper <-   c(5, 5, 4, 4, 5, 2, 2, 3, 3, 1)
frequency_lower <- c(5, 3, 2, 1, 1, 3, 2, 3, 1, 1)
frequency_upper <- c(5, 5, 4, 3, 3, 4, 5, 5, 3, 5)
monetary_lower <-  c(5, 2, 2, 3, 1, 4, 4, 3, 1, 1)
monetary_upper <-  c(5, 5, 4, 5, 5, 5, 5, 5, 4, 5)

segments <- rfm_segment(rfm_result, segment_names, recency_lower, recency_upper,
frequency_lower, frequency_upper, monetary_lower, monetary_upper)

segments %>%
  head(10) %>%
  kable() %>%
  kable_styling()

## ----segment_summary----------------------------------------------------------
rfm_segment_summary(segments)

## ----segment_plot, fig.align='center', fig.width=7, fig.height=7--------------
segments %>%
  rfm_segment_summary() %>%
  rfm_plot_segment()

## ----segment_sumamry_plot, fig.align='center', fig.width=7, fig.height=7------
segments %>%
  rfm_segment_summary() %>%
  rfm_plot_segment_summary()

## ----revenue_dist_plot, fig.align='center', fig.width=7, fig.height=7---------
segments %>%
  rfm_segment_summary() %>%
  rfm_plot_revenue_dist()

#' \code{rfm} package
#'
#' Tools for customer segmentation analysis
#'
#' @docType package
#' @keywords internal
#' @name rfm
#' @aliases rfm-package
"_PACKAGE"

## quiets concerns of R CMD check re: the .'s that appear in pipelines
if (getRversion() >= "2.15.1") {
  utils::globalVariables(c(
    ".", "amount", "score", "segment", "revenue", "orders", "mem",
    "date_most_recent", "frequency_bins", "frequency_score", "monetary",
    "monetary_score", "recency_bins", "recency_days", "recency_score",
    "rfm", "rfm_score", "transaction_count", "n", "category", "share",
    "customers", "customer_share", "revenue_share", "prop", "frame",
    "rfm_result", "customer_id"
  ))
}

#' @export
#' @rdname rfm_table_customer
#' @usage NULL
#'
rfm_table_customer_2 <- function(data = NULL, customer_id = NULL,
                                 n_transactions = NULL, latest_visit_date = NULL,
                                 total_revenue = NULL, analysis_date = NULL,
                                 recency_bins = 5, frequency_bins = 5,
                                 monetary_bins = 5, ...) {

  .Deprecated("rfm_table_customer()")

  col_names <- c("customer_id", "recency_days", "transaction_count", "amount")

  result <-
    data %>%
    mutate(
      recency_days = as.numeric(analysis_date - {{ latest_visit_date }},
                                units = "days")) %>%
    select({{ customer_id }}, recency_days, {{ n_transactions }},
           {{ total_revenue }}) %>%
    set_names(col_names)

  rfm_prep_bins(result, recency_bins, frequency_bins, monetary_bins,
                analysis_date)

}

#' Segmentation
#'
#' Create segments based on recency, frequency and monetary scores.
#'
#' @param data An object of class \code{rfm_table}.
#' @param segment_names Names of the segments.
#' @param recency_lower Lower boundary for recency score.
#' @param recency_upper Upper boundary for recency score.
#' @param frequency_lower Lower boundary for frequency score.
#' @param frequency_upper Upper boundary for frequency score.
#' @param monetary_lower Lower boundary for monetary score.
#' @param monetary_upper Upper boundary for monetary score.
#'
#' @examples
#' # analysis date
#' analysis_date <- as.Date('2006-12-31')
#'
#' # generate rfm score
#' rfm_result <- rfm_table_order(rfm_data_orders, customer_id, order_date,
#' revenue, analysis_date)
#'
#' # segment names
#' segment_names <- c("Champions", "Potential Loyalist", "Loyal Customers",
#'                    "Promising", "New Customers", "Can't Lose Them",
#'                    "At Risk", "Need Attention", "About To Sleep", "Lost")
#'
#' # segment intervals
#' recency_lower <-   c(5, 3, 2, 3, 4, 1, 1, 1, 2, 1)
#' recency_upper <-   c(5, 5, 4, 4, 5, 2, 2, 3, 3, 1)
#' frequency_lower <- c(5, 3, 2, 1, 1, 3, 2, 3, 1, 1)
#' frequency_upper <- c(5, 5, 4, 3, 3, 4, 5, 5, 3, 5)
#' monetary_lower <-  c(5, 2, 2, 3, 1, 4, 4, 3, 1, 1)
#' monetary_upper <-  c(5, 5, 4, 5, 5, 5, 5, 5, 4, 5)
#'
#' # generate segments
#' segments <- rfm_segment(rfm_result, segment_names, recency_lower,
#' recency_upper, frequency_lower, frequency_upper, monetary_lower,
#' monetary_upper)
#'
#' segments
#'
#' @importFrom dplyr between everything
#'
#'@export
#'
rfm_segment <- function(data, segment_names = NULL, recency_lower = NULL,
                        recency_upper = NULL, frequency_lower = NULL,
                        frequency_upper = NULL, monetary_lower = NULL,
                        monetary_upper = NULL) {

  customer_id <- NULL
  segment <- NULL

  rfm_score_table <- data$rfm
  rfm_score_table$segment <- 1

  n_segments <- length(segment_names)

  for (i in seq_len(n_segments)) {
    rfm_score_table$segment[(
      (rfm_score_table$recency_score %>% between(recency_lower[i],
                                                 recency_upper[i])) &
        (rfm_score_table$frequency_score %>% between(frequency_lower[i],
                                                     frequency_upper[i])) &
        (rfm_score_table$monetary_score %>% between(monetary_lower[i],
                                                    monetary_upper[i])) &
        !rfm_score_table$segment %in% segment_names)] <- segment_names[i]
  }

  rfm_score_table$segment[is.na(rfm_score_table$segment)] <- "Others"
  rfm_score_table$segment[rfm_score_table$segment == 1]   <- "Others"

  rfm_score_table[c("customer_id", "segment", "rfm_score", "transaction_count",
                    "recency_days", "amount", "recency_score",
                    "frequency_score", "monetary_score")]
  rfm_score_table %>%
    select(c("customer_id", "segment", "rfm_score", "transaction_count",
                    "recency_days", "amount", "recency_score",
                    "frequency_score", "monetary_score"), everything())


}

#' Segment summary
#'
#' An overview of customer segments.
#'
#' @param segments Output from \code{rfm_segment}.
#'
#' @examples
#' # analysis date
#' analysis_date <- as.Date('2006-12-31')
#'
#' # generate rfm score
#' rfm_result <- rfm_table_order(rfm_data_orders, customer_id, order_date,
#' revenue, analysis_date)
#'
#' # segment names
#' segment_names <- c("Champions", "Potential Loyalist", "Loyal Customers",
#'                    "Promising", "New Customers", "Can't Lose Them",
#'                    "At Risk", "Need Attention", "About To Sleep", "Lost")
#'
#' # segment intervals
#' recency_lower <-   c(5, 3, 2, 3, 4, 1, 1, 1, 2, 1)
#' recency_upper <-   c(5, 5, 4, 4, 5, 2, 2, 3, 3, 1)
#' frequency_lower <- c(5, 3, 2, 1, 1, 3, 2, 3, 1, 1)
#' frequency_upper <- c(5, 5, 4, 3, 3, 4, 5, 5, 3, 5)
#' monetary_lower <-  c(5, 2, 2, 3, 1, 4, 4, 3, 1, 1)
#' monetary_upper <-  c(5, 5, 4, 5, 5, 5, 5, 5, 4, 5)
#'
#' # generate segments
#' segments <- rfm_segment(rfm_result, segment_names, recency_lower,
#' recency_upper, frequency_lower, frequency_upper, monetary_lower,
#' monetary_upper)
#'
#' # segment summary
#' rfm_segment_summary(segments)
#'
#' @export
#'
rfm_segment_summary <- function(segments) {

  segments %>%
    group_by(segment) %>%
    summarise(
      customers = n(),
      orders = sum(transaction_count),
      revenue = sum(amount)
    ) %>%
    mutate(aov = round((revenue / orders), 2))

}

#' Visulaize segment summary
#'
#' Generates plots for customers, orders, revenue and average order value for
#'   each segment.
#'
#' @param x An object of class \code{rfm_segment_summary}.
#' @param metric Metric to be visualized. Defaults to \code{"customers"}. Valid
#'  values are:
#' \itemize{
#' \item \code{"customers"}
#' \item \code{"orders"}
#' \item \code{"revenue"}
#' \item \code{"aov"}
#' }
#' @param sort logical; if \code{TRUE}, sort metrics.
#' @param ascending logical; if \code{TRUE}, sort metrics in ascending order.
#' @param flip logical; if \code{TRUE}, creates horizontal bar plot.
#' @param bar_color Color of the bars.
#' @param plot_title Title of the plot.
#' @param xaxis_label X axis label.
#' @param yaxis_label Y axis label.
#' @param axis_label_size Font size of X axis tick labels.
#' @param axis_label_angle Angle of X axis tick labels.
#' @param bar_labels If \code{TRUE}, add labels to the bars. Defaults to
#'   \code{TRUE}.
#' @param interactive If \code{TRUE}, uses \code{plotly} as the visualization
#'   engine. If \code{FALSE}, uses \code{ggplot2}.
#' @param animate If \code{TRUE}, animates the bars. Defaults to \code{FALSE}.
#' @param print_plot logical; if \code{TRUE}, prints the plot else returns a
#'   plot object.
#'
#' @examples
#' # analysis date
#' analysis_date <- as.Date('2006-12-31')
#'
#' # generate rfm score
#' rfm_result <- rfm_table_order(rfm_data_orders, customer_id, order_date,
#' revenue, analysis_date)
#'
#' # segment names
#' segment_names <- c("Champions", "Potential Loyalist", "Loyal Customers",
#'                    "Promising", "New Customers", "Can't Lose Them",
#'                    "At Risk", "Need Attention", "About To Sleep", "Lost")
#'
#' # segment intervals
#' recency_lower <-   c(5, 3, 2, 3, 4, 1, 1, 1, 2, 1)
#' recency_upper <-   c(5, 5, 4, 4, 5, 2, 2, 3, 3, 1)
#' frequency_lower <- c(5, 3, 2, 1, 1, 3, 2, 3, 1, 1)
#' frequency_upper <- c(5, 5, 4, 3, 3, 4, 5, 5, 3, 5)
#' monetary_lower <-  c(5, 2, 2, 3, 1, 4, 4, 3, 1, 1)
#' monetary_upper <-  c(5, 5, 4, 5, 5, 5, 5, 5, 4, 5)
#'
#' # generate segments
#' segments <- rfm_segment(rfm_result, segment_names, recency_lower,
#' recency_upper, frequency_lower, frequency_upper, monetary_lower,
#' monetary_upper)
#'
#' # segment summary
#' segment_overview <- rfm_segment_summary(segments)
#'
#' # plot segment summary
#' # summarize metric for all segments
#' # ggplot2
#' rfm_plot_segment_summary(segment_overview)
#'
#' # plotly
#' rfm_plot_segment_summary(segment_overview, interactive = TRUE)
#'
#' # select metric to be visualized
#' rfm_plot_segment_summary(segment_overview, metric = "orders")
#'
#' # sort the metric in ascending order
#' rfm_plot_segment_summary(segment_overview, metric = "orders", sort = TRUE,
#'   ascending = TRUE)
#'
#' # default sorting is in descending order
#' rfm_plot_segment_summary(segment_overview, metric = "orders", sort = TRUE)
#'
#' # horizontal bars
#' rfm_plot_segment_summary(segment_overview, metric = "orders", flip = TRUE)
#'
#' @export
#'
rfm_plot_segment_summary <- function(x, metric = NULL,  sort = FALSE,
                                     ascending = FALSE, flip = FALSE,
                                     bar_color = NULL, plot_title = NULL,
                                     xaxis_label = NULL, yaxis_label = NULL,
                                     axis_label_size = 8, axis_label_angle = 315,
                                     bar_labels = TRUE, interactive = FALSE,
                                     animate = FALSE, print_plot = TRUE) {

  if (is.null(metric)) {
    metric <- "customers"
  }

  if (is.null(bar_color)) {
    bar_color <- "#0f1a34"
  }

  if (is.null(plot_title)) {
    plot_title <- paste(to_title_case(metric), " Distribution by Segment")
  }

  if (is.null(xaxis_label)) {
    xaxis_label <- "Segment"
  }

  if (is.null(yaxis_label)) {
    yaxis_label <- to_title_case(metric)
  }

  if (interactive) {
    animate <- FALSE
  }

  data <- x[c("segment", metric)]
  ylim_max <-
    data[[metric]] %>%
    max() %>%
    multiply_by(1.15) %>%
    ceiling(.)

  if (interactive) {
    rfm_plotly_segment_summary(data, metric, flip, sort, ascending, bar_color,
      plot_title, xaxis_label, yaxis_label)
  } else {
    if (animate) {
      print_plot <- FALSE
      data <- rfm_animate_data(data, metric)
    }

    p <- rfm_gg_segment_summary(data, metric, sort, ascending, flip,
                                bar_color, plot_title, xaxis_label,
                                yaxis_label, axis_label_size, axis_label_angle,
                                ylim_max, bar_labels)

    if (animate) {
      p <- rfm_animate_plot(p)
      gganimate::animate(p, fps = 8, renderer = gganimate::gifski_renderer(loop = FALSE))
    }

    if (print_plot) {
      print(p)
    } else {
      return(p)
    }
  }

}

#' Revenue distribution
#'
#' Customer and revenue distribution by segments.
#'
#' @param x An object of class \code{rfm_segment_summary}.
#' @param flip logical; if \code{TRUE}, creates horizontal bar plot.
#' @param colors Bar colors.
#' @param legend_labels Legend labels.
#' @param plot_title Title of the plot.
#' @param xaxis_label X axis label.
#' @param yaxis_label Y axis label.
#' @param axis_label_size Font size of X axis tick labels.
#' @param axis_label_angle Angle of X axis tick labels.
#' @param bar_labels If \code{TRUE}, add labels to the bars. Defaults to
#'   \code{FALSE}.
#' @param bar_label_size Size of bar labels.
#' @param interactive If \code{TRUE}, uses \code{plotly} as the visualization
#'   engine. If \code{FALSE}, uses \code{ggplot2}.
#' @param animate If \code{TRUE}, animates the bars. Defaults to \code{FALSE}.
#' @param print_plot logical; if \code{TRUE}, prints the plot else returns a
#'   plot object.
#'
#' @examples
#' # analysis date
#' analysis_date <- as.Date('2006-12-31')
#'
#' # generate rfm score
#' rfm_result <- rfm_table_order(rfm_data_orders, customer_id, order_date,
#' revenue, analysis_date)
#'
#' # segment names
#' segment_names <- c("Champions", "Potential Loyalist", "Loyal Customers",
#'                    "Promising", "New Customers", "Can't Lose Them",
#'                    "At Risk", "Need Attention", "About To Sleep", "Lost")
#'
#' # segment intervals
#' recency_lower <-   c(5, 3, 2, 3, 4, 1, 1, 1, 2, 1)
#' recency_upper <-   c(5, 5, 4, 4, 5, 2, 2, 3, 3, 1)
#' frequency_lower <- c(5, 3, 2, 1, 1, 3, 2, 3, 1, 1)
#' frequency_upper <- c(5, 5, 4, 3, 3, 4, 5, 5, 3, 5)
#' monetary_lower <-  c(5, 2, 2, 3, 1, 4, 4, 3, 1, 1)
#' monetary_upper <-  c(5, 5, 4, 5, 5, 5, 5, 5, 4, 5)
#'
#' # generate segments
#' segments <- rfm_segment(rfm_result, segment_names, recency_lower,
#' recency_upper, frequency_lower, frequency_upper, monetary_lower,
#' monetary_upper)
#'
#' # segment summary
#' segment_overview <- rfm_segment_summary(segments)
#'
#' # revenue distribution
#' # ggplot2
#' rfm_plot_revenue_dist(segment_overview)
#'
#' # flip
#' rfm_plot_revenue_dist(segment_overview, flip = TRUE)
#'
#' # plotly
#' rfm_plot_revenue_dist(segment_overview, interactive = TRUE)
#'
#' @export
#'
rfm_plot_revenue_dist <- function(x, flip = FALSE,
                                  colors = c("#3b5bdb", "#91a7ff"),
                                  legend_labels = c("Revenue", "Customers"),
                                  plot_title = "Revenue & Customer Distribution",
                                  xaxis_label = NULL, yaxis_label = NULL,
                                  axis_label_size = 8, axis_label_angle = 315,
                                  bar_labels = FALSE, bar_label_size = 2,
                                  interactive = FALSE, animate = FALSE,
                                  print_plot = TRUE) {

  if (interactive) {
    animate <- FALSE
  }

  if (interactive) {
    rfm_plotly_revenue_dist(x, flip, colors, legend_labels, plot_title,
      xaxis_label, yaxis_label)
  } else {
    data <- rfm_prep_revenue_dist(x)
    share_data <- data[, c("category"), drop = FALSE]

    if (animate) {
      print_plot <- FALSE
      bar_labels <- FALSE
      data <- rfm_animate_data(data, "share")
      data$category <- rep(share_data$category, 10)
    }

    p <- rfm_gg_revenue_dist(data, colors, legend_labels, flip,
                             plot_title, xaxis_label, yaxis_label,
                             axis_label_size, axis_label_angle,
                             bar_labels, bar_label_size)

    if (animate) {
      p <- rfm_animate_plot(p)
      gganimate::animate(p, fps=8, renderer = gganimate::gifski_renderer(loop = FALSE))
    }

    if (print_plot) {
      print(p)
    } else {
      return(p)
    }
  }

}

#' Median plots
#'
#' Segment wise median recency, frequency & monetary value plot.
#'
#' @param rfm_segment_table Output from \code{rfm_segment}.
#' @param sort logical; if \code{TRUE}, sort metrics.
#' @param ascending logical; if \code{TRUE}, sort metrics in ascending order.
#' @param flip logical; if \code{TRUE}, creates horizontal bar plot.
#' @param bar_color Color of the bars.
#' @param plot_title Title of the plot.
#' @param xaxis_label X axis label.
#' @param yaxis_label Y axis label.
#' @param axis_label_size Font size of X axis tick labels.
#' @param axis_label_angle Angle of X axis tick labels.
#' @param bar_labels If \code{TRUE}, add labels to the bars. Defaults to
#'   \code{TRUE}.
#' @param interactive If \code{TRUE}, uses \code{plotly} as the visualization
#'   engine. If \code{FALSE}, uses \code{ggplot2}.
#' @param animate If \code{TRUE}, animates the bars. Defaults to \code{FALSE}.
#' @param print_plot logical; if \code{TRUE}, prints the plot else returns a
#'   plot object.
#'
#' @examples
#' # analysis date
#' analysis_date <- as.Date('2006-12-31')
#'
#' # generate rfm score
#' rfm_result <- rfm_table_order(rfm_data_orders, customer_id, order_date,
#' revenue, analysis_date)
#'
#' # segment names
#' segment_names <- c("Champions", "Potential Loyalist", "Loyal Customers",
#'                    "Promising", "New Customers", "Can't Lose Them",
#'                    "At Risk", "Need Attention", "About To Sleep", "Lost")
#'
#' # segment intervals
#' recency_lower <-   c(5, 3, 2, 3, 4, 1, 1, 1, 2, 1)
#' recency_upper <-   c(5, 5, 4, 4, 5, 2, 2, 3, 3, 1)
#' frequency_lower <- c(5, 3, 2, 1, 1, 3, 2, 3, 1, 1)
#' frequency_upper <- c(5, 5, 4, 3, 3, 4, 5, 5, 3, 5)
#' monetary_lower <-  c(5, 2, 2, 3, 1, 4, 4, 3, 1, 1)
#' monetary_upper <-  c(5, 5, 4, 5, 5, 5, 5, 5, 4, 5)
#'
#' # generate segments
#' segments <- rfm_segment(rfm_result, segment_names, recency_lower,
#' recency_upper, frequency_lower, frequency_upper, monetary_lower,
#' monetary_upper)
#'
#' # plots
#' # visualize median recency
#' rfm_plot_median_recency(segments)
#'
#' # plotly
#' rfm_plot_median_recency(segments, interactive = TRUE)
#'
#' # sort in ascending order
#' rfm_plot_median_recency(segments, sort = TRUE, ascending = TRUE)
#'
#' # default sorting is in descending order
#' rfm_plot_median_recency(segments, sort = TRUE)
#'
#' # horizontal bars
#' rfm_plot_median_recency(segments, flip = TRUE)
#'
#' # median frequency
#' rfm_plot_median_frequency(segments)
#'
#' # median monetary value
#' rfm_plot_median_monetary(segments)
#'
#' @export
#'
rfm_plot_median_recency <- function(rfm_segment_table, sort = FALSE,
                                    ascending = FALSE, flip = FALSE,
                                    bar_color = NULL, plot_title = NULL,
                                    xaxis_label = NULL, yaxis_label = NULL,
                                    axis_label_size = 8, axis_label_angle = 315,
                                    bar_labels = TRUE, interactive = FALSE,
                                    animate = FALSE, print_plot = TRUE) {

  if (interactive) {
    animate <- FALSE
  }

  data <- rfm_prep_median(rfm_segment_table, recency_days)

  if (interactive) {
    rfm_plotly_median(data, bar_color, sort, ascending, flip, plot_title,
      xaxis_label, yaxis_label)
  } else {
    if (animate) {
      print_plot <- FALSE
      data <- rfm_animate_data(data, "recency_days")
    }

    p <- rfm_gg_median(data, bar_color, sort, ascending, flip, plot_title,
                       xaxis_label, yaxis_label, axis_label_size,
                       axis_label_angle, bar_labels)

    if (animate) {
      p <- rfm_animate_plot(p)
      gganimate::animate(p, fps=8, renderer = gganimate::gifski_renderer(loop = FALSE))
    }

    if (print_plot) {
      print(p)
    } else {
      return(p)
    }
  }


}

#' @rdname rfm_plot_median_recency
#' @export
#'
rfm_plot_median_frequency <- function(rfm_segment_table, sort = FALSE,
                                      ascending = FALSE, flip = FALSE,
                                      bar_color = NULL, plot_title = NULL,
                                      xaxis_label = NULL, yaxis_label = NULL,
                                      axis_label_size = 8,
                                      axis_label_angle = 315,
                                      bar_labels = TRUE, interactive = FALSE,
                                      animate = FALSE, print_plot = TRUE) {

  if (interactive) {
    animate <- FALSE
  }

  data <- rfm_prep_median(rfm_segment_table, transaction_count)

  if (interactive) {
    rfm_plotly_median(data, bar_color, sort, ascending, flip, plot_title,
      xaxis_label, yaxis_label)
  } else {
    if (animate) {
      print_plot <- FALSE
      data <- rfm_animate_data(data, "transaction_count")
    }

    p <- rfm_gg_median(data, bar_color, sort, ascending, flip, plot_title,
                       xaxis_label, yaxis_label, axis_label_size,
                       axis_label_angle, bar_labels)

    if (animate) {
      p <- rfm_animate_plot(p)
      gganimate::animate(p, fps=8, renderer = gganimate::gifski_renderer(loop = FALSE))
    }

    if (print_plot) {
      print(p)
    } else {
      return(p)
    }
  }

}


#' @rdname rfm_plot_median_recency
#' @export
#'
rfm_plot_median_monetary <- function(rfm_segment_table, sort = FALSE,
                                     ascending = FALSE, flip = FALSE,
                                     bar_color = NULL, plot_title = NULL,
                                     xaxis_label = NULL, yaxis_label = NULL,
                                     axis_label_size = 8,
                                     axis_label_angle = 315,
                                     bar_labels = TRUE, interactive = FALSE,
                                     animate = FALSE, print_plot = TRUE) {

  if (interactive) {
    animate <- FALSE
  }

  data <- rfm_prep_median(rfm_segment_table, amount)

  if (interactive) {
    rfm_plotly_median(data, bar_color, sort, ascending, flip, plot_title,
      xaxis_label, yaxis_label)
  } else {

    if (animate) {
      print_plot <- FALSE
      data <- rfm_animate_data(data, "amount")
    }

    p <- rfm_gg_median(data, bar_color, sort, ascending, flip, plot_title,
                       xaxis_label, yaxis_label, axis_label_size, axis_label_angle,
                       bar_labels)

    if (animate) {
      p <- rfm_animate_plot(p)
      gganimate::animate(p, fps=8, renderer = gganimate::gifski_renderer(loop = FALSE))
    }

    if (print_plot) {
      print(p)
    } else {
      return(p)
    }
  }

}

#' RFM Segmentation Plot
#'
#' Generates tree map to visualize segments.
#'
#' @param table An object of class \code{rfm_segment_summary}.
#' @param metric Metric to be visualized. Defaults to \code{"customers"}. Valid
#' values are:
#' \itemize{
#' \item \code{"customers"}
#' \item \code{"orders"}
#' \item \code{"revenue"}
#' }
#' @param interactive If \code{TRUE}, uses \code{plotly} as the visualization
#'   engine. If \code{FALSE}, uses \code{ggplot2}.
#' @param print_plot logical; if \code{TRUE}, prints the plot else returns a
#'   plot object.
#'
#' @examples
#' # analysis date
#' analysis_date <- as.Date('2006-12-31')
#'
#' # generate rfm score
#' rfm_result <- rfm_table_order(rfm_data_orders, customer_id, order_date,
#' revenue, analysis_date)
#'
#' # segment names
#' segment_names <- c("Champions", "Potential Loyalist", "Loyal Customers",
#'                    "Promising", "New Customers", "Can't Lose Them",
#'                    "At Risk", "Need Attention", "About To Sleep", "Lost")
#'
#' # segment intervals
#' recency_lower <-   c(5, 3, 2, 3, 4, 1, 1, 1, 2, 1)
#' recency_upper <-   c(5, 5, 4, 4, 5, 2, 2, 3, 3, 1)
#' frequency_lower <- c(5, 3, 2, 1, 1, 3, 2, 3, 1, 1)
#' frequency_upper <- c(5, 5, 4, 3, 3, 4, 5, 5, 3, 5)
#' monetary_lower <-  c(5, 2, 2, 3, 1, 4, 4, 3, 1, 1)
#' monetary_upper <-  c(5, 5, 4, 5, 5, 5, 5, 5, 4, 5)
#'
#' # generate segments
#' segments <- rfm_segment(rfm_result, segment_names, recency_lower,
#' recency_upper, frequency_lower, frequency_upper, monetary_lower,
#' monetary_upper)
#'
#' # segment summary
#' segment_overview <- rfm_segment_summary(segments)
#'
#' # treemaps
#' # default metric is customers
#' rfm_plot_segment(segment_overview)
#'
#' # treemap of orders
#' rfm_plot_segment(segment_overview, metric = "orders")
#'
#' # plotly
#' rfm_plot_segment(segment_overview, metric = "revenue", interactive = TRUE)
#'
#' @export
#'
rfm_plot_segment <- function(table, metric = "customers", interactive = FALSE,
                             print_plot = TRUE) {

  table$prop <- round((table[[metric]] / sum(table[[metric]])) * 100, 2)

  if (interactive) {
    p <- rfm_plotly_segment(table, metric)
  } else {
    p <- rfm_gg_segment(table, metric)
  }

  if (print_plot) {
    print(p)
  } else {
    return(p)
  }

}

#' Segment Scatter Plots
#'
#' @description Generate scatter plots to examine the relationship between
#' recency, frequency and monetary value.
#'
#' @param segments Output from \code{rfm_segment}.
#' @param x Metric to be represented on X axis.
#' @param y Metric to be represented on Y axis.
#' @param plot_title Title of the plot.
#' @param legend_title Title of the plot legend.
#' @param xaxis_label X axis label.
#' @param yaxis_label Y axis label.
#' @param interactive If \code{TRUE}, uses \code{plotly} as the visualization
#'   engine. If \code{FALSE}, uses \code{ggplot2}.
#' @param animate If \code{TRUE}, animates the bars. Defaults to \code{FALSE}.
#' @param print_plot logical; if \code{TRUE}, prints the plot else returns a
#'   plot object.
#'
#' @return Scatter plot.
#'
#' @examples
#' # analysis date
#' analysis_date <- as.Date('2006-12-31')
#'
#' # generate rfm score
#' rfm_result <- rfm_table_order(rfm_data_orders, customer_id, order_date,
#' revenue, analysis_date)
#'
#' # segment names
#' segment_names <- c("Champions", "Potential Loyalist", "Loyal Customers",
#'                    "Promising", "New Customers", "Can't Lose Them",
#'                    "At Risk", "Need Attention", "About To Sleep", "Lost")
#'
#' # segment intervals
#' recency_lower <-   c(5, 3, 2, 3, 4, 1, 1, 1, 2, 1)
#' recency_upper <-   c(5, 5, 4, 4, 5, 2, 2, 3, 3, 1)
#' frequency_lower <- c(5, 3, 2, 1, 1, 3, 2, 3, 1, 1)
#' frequency_upper <- c(5, 5, 4, 3, 3, 4, 5, 5, 3, 5)
#' monetary_lower <-  c(5, 2, 2, 3, 1, 4, 4, 3, 1, 1)
#' monetary_upper <-  c(5, 5, 4, 5, 5, 5, 5, 5, 4, 5)
#'
#' # generate segments
#' segments <- rfm_segment(rfm_result, segment_names, recency_lower,
#' recency_upper, frequency_lower, frequency_upper, monetary_lower,
#' monetary_upper)
#'
#' # visualize
#' # ggplot2
#' rfm_plot_segment_scatter(segments, "monetary", "recency")
#'
#' # plotly
#' rfm_plot_segment_scatter(segments, "monetary", "recency", interactive = TRUE)
#'
#' @export
rfm_plot_segment_scatter <- function(segments, x = "monetary", y = "recency",
                                     plot_title = NULL, legend_title = NULL,
                                     xaxis_label = NULL, yaxis_label = NULL,
                                     interactive = FALSE, animate = FALSE,
                                     print_plot = TRUE) {

  x_data <- switch(x,
                   "recency" = "recency_days",
                   "frequency" = "transaction_count",
                   "monetary" = "amount"
  )

  y_data <- switch(y,
                   "recency" = "recency_days",
                   "frequency" = "transaction_count",
                   "monetary" = "amount"
  )

  if (is.null(xaxis_label)) {
    xaxis_label <- to_title_case(x)
    if (grepl("Monetary", xaxis_label)) {
      xaxis_label <- paste(xaxis_label, "Value")
    }
  }

  if (is.null(yaxis_label)) {
    yaxis_label <- to_title_case(y)
    if (grepl("Monetary", yaxis_label)) {
      yaxis_label <- paste(yaxis_label, "Value")
    }
  }

  if (is.null(plot_title)) {
    plot_title <- paste(yaxis_label, "vs", xaxis_label)
  }

  if (interactive) {
    animate <- FALSE
  }

  if (interactive) {
    rfm_plotly_segment_scatter(segments, x_data, y_data, plot_title,
      legend_title, xaxis_label, yaxis_label)
  } else {
    p <- rfm_gg_segment_scatter(segments, x_data, y_data, plot_title,
                                legend_title, xaxis_label, yaxis_label)

    if (animate) {
      print_plot <- FALSE
      p <-
        p +
        gganimate::transition_reveal(along = rfm_score)

      gganimate::animate(p, fps=8, renderer = gganimate::gifski_renderer(loop = FALSE))
    }

    if (print_plot) {
      print(p)
    } else {
      return(p)
    }
  }

}

#' RFM transaction data
#'
#' A dataset containing transactions of different customers.
#'
#' @format A tibble with 49.6 rows and 3 variables:
#' \describe{
#'   \item{order_date}{Order date.}
#'   \item{customer_id}{Customer id}
#'   \item{revenue}{Transaction amount.}
#'   \item{first_name}{First name of the customer.}
#'   \item{last_name}{Last name of the customer.}
#'   \item{email}{email id of the customer.}
#' }
#'
"rfm_data_orders"

#' RFM customer data
#'
#' A dataset containing customer level data.
#'
#' @format A tibble with 39,999 rows and 5 variables:
#' \describe{
#'   \item{customer_id}{Customer id.}
#'   \item{total_amount}{Total amount of all orders.}
#'   \item{most_recent_visit}{Date of the most recent transaction.}
#'   \item{number_of_purchases}{Total number of transactions/orders.}
#'   \item{purchase_interval}{Number of days since last transaction/order.}
#'   #'   \item{first_name}{First name of the customer.}
#'   \item{last_name}{Last name of the customer.}
#'   \item{email}{email id of the customer.}
#' }
#'
"rfm_data_customer"

rfm_animate_data <- function(data, metric) {
  n_groups <- length(data$segment)

  gap <- data[[metric]] / 9
  start <- rep(0, n_groups)
  value <- start
  for (i in seq_len(9)) {
    n <- start + gap
    value <- c(value, n)
    start <- n
  }

  result <- data.frame(segment = rep(data$segment, 10),
                       n = round(value, 2),
                       frame = rep(letters[1:10], each = n_groups))

  names(result) <- c("segment", metric, "frame")
  result
}

rfm_animate_plot <- function(p) {
  p +
    gganimate::transition_states(
      frame,
      transition_length = 10,
      state_length = 1,
      wrap = FALSE
    ) +
    gganimate::ease_aes('linear')
}

.onAttach <- function(...) {

  if (!interactive() || runif(1) > 0.1) return()

  pkgs <- available.packages()

  cran_version <-
    pkgs %>%
    extract("rfm", "Version") %>%
    package_version()

  local_version <- packageVersion("rfm")
  behind_cran <- cran_version > local_version

  tips <- c(
    "Learn more about rfm at https://github.com/rsquaredacademy/rfm/.",
    "Use suppressPackageStartupMessages() to eliminate package startup messages.",
    "Need help getting started with regression models? Visit: https://www.rsquaredacademy.com",
    "Check out our interactive app for quick data exploration. Visit: https://apps.rsquaredacademy.com/."
  )

  tip <- sample(tips, 1)

  if (interactive()) {
    if (behind_cran) {
      msg <- c("A new version of rfm is available with bug fixes and new features.")
      packageStartupMessage(msg, "\nWould you like to install it?")
      if (menu(c("Yes", "No")) == 1) {
        update.packages("rfm")
      }
    } else {
      packageStartupMessage(paste(strwrap(tip), collapse = "\n"))
    }
  }

}

#' RFM heatmap
#'
#' @description The heat map shows the average monetary value for different
#'   categories of recency and frequency scores. Higher scores of frequency and
#'   recency are characterized by higher average monetary value as indicated by
#'   the darker areas in the heatmap.
#'
#' @param data An object of class \code{rfm_table}.
#' @param brewer_n Indicates the number of colors in the palette; RColorBrewer
#'   is used for the color palette of the heatmap; check the documentation of
#'   \code{brewer.pal}.
#' @param brewer_name Palette name; check the documentation of
#'   \code{brewer.pal}.
#' @param xaxis_label X axis label.
#' @param yaxis_label Y axis label.
#' @param plot_title Title of the plot.
#' @param legend_title Legend title.
#' @param interactive If \code{TRUE}, uses \code{plotly} as the visualization
#'   engine. If \code{FALSE}, uses \code{ggplot2}.
#' @param print_plot logical; if \code{TRUE}, prints the plot else returns a
#'   plot object.
#'
#' @section Deprecated Functions:
#' \code{rfm_heatmap()} has been deprecated and will be made defunct. It has
#' been provided for compatibility with older versions only, and will be made
#' defunct at the next release.
#'
#' Instead use the replacement function \code{rfm_plot_heatmap()}.
#'
#' @examples
#' # using transaction data
#' analysis_date <- as.Date('2006-12-31')
#' rfm_order <- rfm_table_order(rfm_data_orders, customer_id, order_date,
#' revenue, analysis_date)
#'
#' # heat map
#' # ggplot2
#' rfm_plot_heatmap(rfm_order)
#'
#' # plotly
#' rfm_plot_heatmap(rfm_order, interactive = TRUE)
#'
#' # using customer data
#' analysis_date <- as.Date('2007-01-01')
#' rfm_customer <- rfm_table_customer(rfm_data_customer, customer_id,
#' number_of_orders, recency_days, revenue, analysis_date)
#'
#' # heat map
#' rfm_plot_heatmap(rfm_customer)
#'
#' @export
#'
rfm_plot_heatmap <- function(data, brewer_n = 5, brewer_name = "PuBu",
                             xaxis_label = NULL, yaxis_label = NULL,
                             plot_title = NULL, legend_title = NULL,
                             interactive = FALSE, print_plot = TRUE) {

  mapdata <- rfm_heatmap_data(rfm_table = data)

  if (is.null(legend_title)) {
    legend_title <- "Mean Monetary Value"
  }

  if (is.null(plot_title)) {
    plot_title <- "RFM Heat Map"
  }

  if (is.null(xaxis_label)) {
    xaxis_label <- "Frequency Score"
  }

  if (is.null(yaxis_label)) {
    yaxis_label <- "Recency Score"
  }

  if (interactive) {
    p <- rfm_plotly_heatmap(mapdata, plot_title, xaxis_label, yaxis_label, brewer_n,
                            brewer_name, legend_title)
  } else {
    p <- rfm_gg_heatmap(mapdata, plot_title, xaxis_label, yaxis_label, brewer_n,
                   brewer_name, legend_title)
  }

  if (print_plot) {
    print(p)
  } else {
    return(p)
  }

}

#' @export
#' @rdname rfm_plot_heatmap
#' @usage NULL
#'
rfm_heatmap <- function(data, plot_title = "RFM Heat Map",
                        xaxis_title = "Frequency",
                        yaxis_title = "Recency",
                        legend_title = "Mean Monetary Value",
                        brewer_n = 5, brewer_name = "PuBu",
                        print_plot = TRUE) {
  .Deprecated("rfm_plot_heatmap()")
  rfm_plot_heatmap(data, plot_title = plot_title,
                   xaxis_label = xaxis_title,
                   yaxis_label = yaxis_title,
                   legend_title = legend_title,
                   brewer_n = brewer_n, brewer_name = brewer_name,
                   print_plot = print_plot)
}


#' RFM histograms
#'
#' Histograms of recency, frequency and monetary value.
#'
#' @param rfm_table An object of class \code{rfm_table}.
#' @param metric Metric to be visualized. Defaults to \code{"recency"}. Valid
#'  values are:
#' \itemize{
#' \item \code{"recency"}
#' \item \code{"frequency"}
#' \item \code{"monetary"}
#' }
#' @param hist_bins Number of bins of the histograms.
#' @param hist_color Color of the histogram.
#' @param plot_title Title of the plot.
#' @param xaxis_label X axis label.
#' @param yaxis_label Y axis label.
#' @param interactive If \code{TRUE}, uses \code{plotly} as the visualization
#'   engine. If \code{FALSE}, uses \code{ggplot2}.
#' @param print_plot logical; if \code{TRUE}, prints the plot else returns a
#' plot object.
#'
#' @return Histograms
#'
#' @section Deprecated Functions:
#' \code{rfm_histograms()} has been deprecated and will be made defunct. It has
#' been provided for compatibility with older versions only, and will be made
#' defunct at the next release.
#'
#' Instead use the replacement function \code{rfm_plot_histogram()}.
#'
#' @examples
#' # using transaction data
#' analysis_date <- as.Date('2006-12-31')
#' rfm_order <- rfm_table_order(rfm_data_orders, customer_id, order_date,
#' revenue, analysis_date)
#'
#' # histogram
#' # ggplot2
#' rfm_plot_histogram(rfm_order, metric = "frequency")
#'
#' # plotly
#' rfm_plot_histogram(rfm_order, metric = "frequency", interactive = TRUE)
#'
#' # using customer data
#' analysis_date <- as.Date('2007-01-01')
#' rfm_customer <- rfm_table_customer(rfm_data_customer, customer_id,
#' number_of_orders, recency_days, revenue, analysis_date)
#'
#' # histogram
#' rfm_plot_histogram(rfm_customer)
#'
#' @export
#'
rfm_plot_histogram <- function(rfm_table, metric = "recency",
                               hist_bins = 9, hist_color = NULL,
                               plot_title = NULL, xaxis_label = NULL,
                               yaxis_label = NULL, interactive = FALSE,
                               print_plot = TRUE) {

  if (is.null(xaxis_label)) {
    xaxis_label <- to_title_case(metric)
    if (grepl("Monetary", xaxis_label)) {
      xaxis_label <- paste(xaxis_label, "Value")
    }
  }

  if (is.null(plot_title)) {
    plot_title <- paste(xaxis_label, " Distribution")
  }

  if (is.null(yaxis_label)) {
    yaxis_label <- "Count"
  }

  if (is.null(hist_color)) {
    hist_color <- "#0f1a34"
  }

  ycol <- switch(metric,
                 "recency"   = "recency_days",
                 "frequency" = "transaction_count",
                 "monetary"  = "amount"
  )

  data <- rfm_table$rfm[, ycol, drop = FALSE]
  names(data) <- c("score")

  if (interactive) {
    p <- rfm_plotly_hist(data, hist_color, plot_title, xaxis_label, yaxis_label)
  } else {
    p <- rfm_gg_hist(data, hist_bins, hist_color, plot_title, xaxis_label,
                     yaxis_label)
  }

  if (print_plot) {
    print(p)
  } else {
    return(p)
  }

}

#' @export
#' @rdname rfm_plot_histogram
#' @usage NULL
#'
rfm_histograms <- function(rfm_table, hist_bins = 9, hist_color = "#0f1a34",
                           plot_title = 'RFM Histograms', xaxis_title = ' ',
                           yaxis_title = 'Count', print_plot = TRUE) {
  .Deprecated("rfm_plot_histograms()")
  rfm_plot_histogram(rfm_table, metric = "recency",
                     hist_bins = hist_bins, hist_color = hist_color,
                     plot_title = plot_title, xaxis_label = xaxis_title,
                     yaxis_label = xaxis_title, interactive = FALSE,
                     print_plot = print_plot)
}


#' RFM bar chart
#'
#' @description Examine the distribution of monetary scores for the different
#'   combinations of frequency and recency scores.
#'
#' @param rfm_table An object of class \code{rfm_table}.
#' @param bar_color Color of the bars.
#' @param xaxis_label X axis label.
#' @param yaxis_label Y axis label.
#' @param sec_xaxis_label Secondary x axis label.
#' @param sec_yaxis_label Secondary y axis label.
#' @param print_plot logical; if \code{TRUE}, prints the plot else returns a
#'   plot object.
#'
#' @return Bar chart.
#'
#' @section Deprecated Functions:
#' \code{rfm_bar_chart()} has been deprecated and will be made defunct. It has
#' been provided for compatibility with older versions only, and will be made
#' defunct at the next release.
#'
#' Instead use the replacement function \code{rfm_plot_bar_chart()}.
#'
#' @examples
#' # using transaction data
#' analysis_date <- as.Date('2006-12-31')
#' rfm_order <- rfm_table_order(rfm_data_orders, customer_id, order_date,
#' revenue, analysis_date)
#'
#' # bar chart
#' rfm_plot_bar_chart(rfm_order)
#'
#' @importFrom ggplot2 sec_axis facet_grid
#'
#' @export
#'
rfm_plot_bar_chart <- function(rfm_table, bar_color = NULL,
                               xaxis_label = NULL, sec_xaxis_label = NULL,
                               yaxis_label = NULL, sec_yaxis_label = NULL,
                               print_plot = TRUE) {

  if (is.null(bar_color)) {
    bar_color <- "#0f1a34"
  }

  if (is.null(xaxis_label)) {
    xaxis_label <- "Monetary Score"
  }

  if (is.null(yaxis_label)) {
    yaxis_label <- " "
  }

  if (is.null(sec_xaxis_label)) {
    sec_xaxis_label <- "Frequency Score"
  }

  if (is.null(sec_yaxis_label)) {
    sec_yaxis_label <- "Recency Score"
  }

  p <-
    rfm_barchart_data(rfm_table) %>%
    ggplot() +
    geom_bar(aes(x = monetary_score), fill = bar_color) +
    xlab(xaxis_label) +
    ylab(yaxis_label) +
    ggtitle(sec_xaxis_label) +
    scale_y_continuous(sec.axis = sec_axis(~ ., name = sec_yaxis_label)) +
    facet_grid(recency_score ~ frequency_score) +
    theme(plot.title = element_text(face = "plain", size = 11, hjust = 0.5),
          axis.text.y = element_blank(),
          axis.ticks.y = element_blank())

  if (print_plot) {
    print(p)
  } else {
    return(p)
  }

}

#' @export
#' @rdname rfm_plot_bar_chart
#' @usage NULL
#'
rfm_bar_chart <- function(rfm_table, bar_color = '#0f1a34',
                          xaxis_title = 'Monetary Score',
                          sec_xaxis_title = 'Frequency Score',
                          yaxis_title = ' ',
                          sec_yaxis_title = 'Recency Score',
                          print_plot = TRUE) {
  .Deprecated("rfm_plot_bar_chart()")
  rfm_plot_bar_chart(rfm_table, bar_color = bar_color,
                     xaxis_label = xaxis_title,
                     sec_xaxis_label = sec_xaxis_title,
                     yaxis_label = yaxis_title,
                     sec_yaxis_label = sec_yaxis_title,
                     print_plot = print_plot)
}


#' Customers by orders
#'
#' Visualize the distribution of customers across orders.
#'
#' @param rfm_table An object of class \code{rfm_table}.
#' @param flip logical; if \code{TRUE}, creates horizontal bar plot.
#' @param bar_color Color of the bars.
#' @param plot_title Title of the plot.
#' @param xaxis_label X axis title.
#' @param yaxis_label Y axis title.
#' @param bar_label_size Size of bar labels.
#' @param bar_labels If \code{TRUE}, add labels to the bars. Defaults to
#'   \code{TRUE}.
#' @param interactive If \code{TRUE}, uses \code{plotly} as the visualization
#'   engine. If \code{FALSE}, uses \code{ggplot2}.
#' @param animate If \code{TRUE}, animates the bars. Defaults to \code{FALSE}.
#' @param print_plot logical; if \code{TRUE}, prints the plot else returns a
#'   plot object.
#'
#' @return Bar chart.
#'
#' @section Deprecated Functions:
#' \code{rfm_order_dist()} has been deprecated and will be made defunct. It has
#' been provided for compatibility with older versions only, and will be made
#' defunct at the next release.
#'
#' Instead use the replacement function \code{rfm_plot_order_dist()}.
#'
#' @examples
#' # using transaction data
#' analysis_date <- as.Date('2006-12-31')
#' rfm_order <- rfm_table_order(rfm_data_orders, customer_id, order_date,
#' revenue, analysis_date)
#'
#' # order distribution
#' rfm_plot_order_dist(rfm_order)
#'
#' # horizontal bars
#' rfm_plot_order_dist(rfm_order, flip = TRUE)
#'
#' # plotly
#' rfm_plot_order_dist(rfm_order, interactive = TRUE)
#'
#' # using customer data
#' analysis_date <- as.Date('2007-01-01')
#' rfm_customer <- rfm_table_customer(rfm_data_customer, customer_id,
#' number_of_orders, recency_days, revenue, analysis_date)
#'
#' # order distribution
#' rfm_plot_order_dist(rfm_customer)
#'
#' @export
#'
rfm_plot_order_dist <- function(rfm_table, flip = FALSE, bar_color = NULL,
                                plot_title = NULL, xaxis_label = NULL,
                                yaxis_label = NULL, bar_label_size = 3,
                                bar_labels = TRUE, interactive = FALSE,
                                animate = FALSE, print_plot = TRUE) {

  if (is.null(plot_title)) {
    plot_title <- "Customer Distribution by Orders"
  }

  if (is.null(xaxis_label)) {
    xaxis_label <- "Orders"
  }

  if (is.null(yaxis_label)) {
    yaxis_label <- "Customers"
  }

  if (is.null(bar_color)) {
    bar_color <- "#0f1a34"
  }

  if (interactive) {
    animate <- FALSE
  }

  data <- rfm_order_dist_data(rfm_table)
  ylim_max <- rfm_order_dist_ylim(data)

  if (interactive) {
    colnames(data) <- c("transaction_count", "n")
    rfm_plotly_order_dist(data, flip = flip, bar_color = bar_color,
      plot_title = plot_title, xaxis_label = xaxis_label,
      yaxis_label = yaxis_label)
  } else {
    if (animate) {
      print_plot <- FALSE
      data <- rfm_animate_data(data, "n")
    }

    p <- rfm_gg_order_dist(data, flip, bar_color, plot_title, xaxis_label,
                           yaxis_label, ylim_max, bar_labels, bar_label_size)

    if (animate) {
      p <- rfm_animate_plot(p)
      gganimate::animate(p, fps=8,
                         renderer = gganimate::gifski_renderer(loop = FALSE))
    }

    if (print_plot) {
      print(p)
    } else {
      return(p)
    }
  }

}

#' @export
#' @rdname rfm_plot_order_dist
#' @usage NULL
#'
rfm_order_dist <- function(rfm_table, bar_color = '#0f1a34',
                           xaxis_title = 'Orders', yaxis_title = 'Customers',
                           plot_title = 'Customers by Orders',
                           print_plot = TRUE) {
  .Deprecated("rfm_plot_order_dist()")
  rfm_plot_order_dist(rfm_table, bar_color = bar_color,
                      xaxis_label = xaxis_title, yaxis_label = yaxis_title,
                      plot_title = plot_title,
                      print_plot = print_plot)
}


#' RFM Scatter plot
#'
#' @description Examine the relationship between recency, frequency
#'   and monetary values.
#'
#' @param segments Output from \code{rfm_segment}.
#' @param xaxis_label X axis label.
#' @param yaxis_label Y axis label.
#' @param plot_title Title of the plot.
#' @param print_plot logical; if \code{TRUE}, prints the plot else returns a
#'   plot object.
#'
#' @return Scatter plot.
#'
#' @section Deprecated Functions:
#' \code{rfm_rm_plot()}, \code{rfm_fm_plot()} and \code{rfm_rf_plot()} have
#' been deprecated and will be made defunct. These functions have been provided
#' for compatibility with older versions only, and will be made defunct at the
#' next release.
#' Instead use the replacement function \code{rfm_plot_segment_scatter()}.
#'
#' @export
#'
rfm_rm_plot <- function(segments, xaxis_label = NULL,
                        yaxis_label = NULL, plot_title = NULL,
                        print_plot = TRUE) {

  .Deprecated("rfm_plot_scatter")
  rfm_plot_segment_scatter(segments, "monetary", "recency", xaxis_label,
                           yaxis_label, plot_title, print_plot)

}

#' @rdname rfm_rm_plot
#' @export
#'
rfm_fm_plot <- function(segments, xaxis_label = NULL,
                        yaxis_label = NULL, plot_title = NULL,
                        print_plot = TRUE) {

  .Deprecated("rfm_plot_scatter")
  rfm_plot_segment_scatter(segments, "monetary", "frequency", xaxis_label,
                           yaxis_label, plot_title, print_plot)

}

#' @rdname rfm_rm_plot
#' @export
#'
rfm_rf_plot <- function(segments, xaxis_label = NULL,
                        yaxis_label = NULL, plot_title = NULL,
                        print_plot = TRUE) {

  .Deprecated("rfm_plot_scatter")
  rfm_plot_segment_scatter(segments, "frequency", "recency", xaxis_label,
                           yaxis_label, plot_title, print_plot)

}

#' Heatmap data
#'
#' Data for generating heatmap.
#'
#' @param rfm_table An object of class \code{rfm_table}.
#'
#' @examples
#' # using transaction data
#' analysis_date <- as.Date('2006-12-31')
#' rfm_order <- rfm_table_order(rfm_data_orders, customer_id, order_date,
#' revenue, analysis_date)
#'
#' # heat map data
#' rfm_heatmap_data(rfm_order)
#'
#' # using customer data
#' analysis_date <- as.Date('2007-01-01')
#' rfm_customer <- rfm_table_customer(rfm_data_customer, customer_id,
#' number_of_orders, recency_days, revenue, analysis_date)
#'
#' # heat map data
#' rfm_heatmap_data(rfm_customer)
#'
#' @importFrom magrittr use_series
#' @importFrom dplyr group_by summarise
#'
#' @export
#'
rfm_heatmap_data <- function(rfm_table) {

  result <-
    rfm_table %>%
    use_series(rfm) %>%
    group_by(frequency_score, recency_score) %>%
    select(frequency_score, recency_score, amount) %>%
    summarise(monetary = mean(amount))

  l_frequency      <- check_levels(result, "frequency_score")
  l_recency        <- check_levels(result, "recency_score")
  levels_frequency <- length(l_frequency)
  levels_recency   <- length(l_recency)
  f_frequency      <- use_series(rfm_table, frequency_bins)
  r_recency        <- use_series(rfm_table, recency_bins)

  if (levels_frequency != f_frequency) {
    result %<>%
      modify_rfm(., f_frequency, l_frequency)
  }

  if (levels_recency != r_recency) {
    result %<>%
      modify_rfm(., r_recency, l_recency)
  }

  return(result)

}

#' Bar chart data
#'
#' Data for generating bar charts.
#'
#' @param rfm_table An object of class \code{rfm_table}.
#'
#' @examples
#' # using transaction data
#' analysis_date <- as.Date('2006-12-31')
#' rfm_order <- rfm_table_order(rfm_data_orders, customer_id, order_date,
#' revenue, analysis_date)
#'
#' # bar chart data
#' rfm_barchart_data(rfm_order)
#'
#' # using customer data
#' analysis_date <- as.Date('2007-01-01')
#' rfm_customer <- rfm_table_customer(rfm_data_customer, customer_id,
#' number_of_orders, recency_days, revenue, analysis_date)
#'
#' # bar chart data
#' rfm_barchart_data(rfm_customer)
#'
#' @export
#'
rfm_barchart_data <- function(rfm_table) {

  rlevels <-
    rfm_table %>%
    use_series(recency_bins) %>%
    seq_len(.) %>%
    rev()

  data <- use_series(rfm_table, rfm)
  data$recency_score <- factor(data$recency_score, levels = rlevels)

  return(data)

}

#' @importFrom dplyr rename
rfm_order_dist_data <- function(rfm_table) {
  rfm_table %>%
    use_series(rfm) %>%
    group_by(transaction_count) %>%
    summarise(n = n()) %>%
    rename(segment = transaction_count)
}

#' @importFrom magrittr multiply_by
rfm_order_dist_ylim <- function(data) {
  data %>%
    use_series(n) %>%
    max() %>%
    multiply_by(1.1) %>%
    ceiling(.)
}

rfm_prep_revenue_dist <- function(x) {

  x$customer_share <- x$customers / sum(x$customers)
  x$revenue_share <- x$revenue / sum(x$revenue)
  data <- x[c("segment", "customer_share", "revenue_share")]

  n_row    <- nrow(data)
  segment  <- rep(data$segment, each = 2)
  category <- rep(c("customer_share", "revenue_share"), times = n_row)

  share <- c()
  for (i in seq_len(n_row)) {
    y <- as.numeric(data[i, c(2, 3)])
    share <- c(share, y)
  }

  result <- data.frame(segment, category, share)
  result$category <- factor(result$category,
                            levels = c("revenue_share", "customer_share"))
  return(result)
}

#' @importFrom dplyr arrange
#' @importFrom rlang :=
rfm_prep_median <- function(rfm_segment_table, metric) {

  rfm_segment_table %>%
    select(segment, {{ metric }}) %>%
    group_by(segment) %>%
    summarise({{ metric }} := median({{ metric }})) %>%
    arrange({{ metric }})

}

#' @importFrom magrittr extract add
#' @importFrom stats median runif quantile reorder
#' @importFrom utils available.packages menu update.packages packageVersion
#' @importFrom utils install.packages
bins <- function(data, value, n_bins) {

  length_out <- n_bins + 1

  data[[value]] %>%
    quantile(probs = seq(0, 1, length.out = length_out)) %>%
    unname() %>%
    extract(c(-1, -length_out)) %>%
    add(1)

}

bins_lower <- function(data, value, bins) {

  data[[value]] %>%
    min() %>%
    append(bins)

}

bins_upper <- function(data, value, bins) {

  data_max <-
    data[[value]] %>%
    max() %>%
    add(1)

  c(bins, data_max)
}


check_levels <- function(rfm_heatmap_data, column) {

  rfm_heatmap_data[[column]] %>%
    as.factor() %>%
    levels() %>%
    as.vector() %>%
    as.integer()

}

modify_rfm <- function(rfm_heatmap_data, n_bins, check_levels) {

  missing           <- !(seq_len(n_bins) %in% check_levels)
  missing2          <- seq_len(n_bins)[missing]
  extra_data        <- expand.grid(missing2, seq_len(n_bins), 0)
  names(extra_data) <- names(rfm_heatmap_data)
  rbind(rfm_heatmap_data, extra_data)

}

try_pkg <- function(pkg) {
  tryCatch(packageVersion(pkg), error = function(e) NA)
}

to_title_case <- function(x) {
    s <- strsplit(x, " ")[[1]]
    paste(toupper(substring(s, 1, 1)), substring(s, 2),
          sep = "", collapse = " ")
}

#' RFM report
#'
#' Generates a segmentation analysis report.
#'
#' @param rfm_table An object of class \code{rfm_table}.
#' @param segments Output from \code{rfm_segment}.
#' @param interactive If \code{TRUE}, uses \code{plotly} as the visualization
#'   engine. If \code{FALSE}, uses \code{ggplot2}.
#' @param title Title of the report.
#' @param author Author of the report.
#' @param folder_name The output directory for the report.
#' @param file_name The name of the report file.
#'
#' @examples
#' \dontrun{
#' # analysis date
#' analysis_date <- as.Date('2006-12-31')
#'
#' # generate rfm score
#' rfm_result <- rfm_table_order(rfm_data_orders, customer_id, order_date,
#' revenue, analysis_date)
#'
#' # segment names
#' segment_names <- c("Champions", "Potential Loyalist", "Loyal Customers",
#'                    "Promising", "New Customers", "Can't Lose Them",
#'                    "At Risk", "Need Attention", "About To Sleep", "Lost")
#'
#' # segment intervals
#' recency_lower <-   c(5, 3, 2, 3, 4, 1, 1, 1, 2, 1)
#' recency_upper <-   c(5, 5, 4, 4, 5, 2, 2, 3, 3, 1)
#' frequency_lower <- c(5, 3, 2, 1, 1, 3, 2, 3, 1, 1)
#' frequency_upper <- c(5, 5, 4, 3, 3, 4, 5, 5, 3, 5)
#' monetary_lower <-  c(5, 2, 2, 3, 1, 4, 4, 3, 1, 1)
#' monetary_upper <-  c(5, 5, 4, 5, 5, 5, 5, 5, 4, 5)
#'
#' # generate segments
#' segments <- rfm_segment(rfm_result, segment_names, recency_lower,
#' recency_upper, frequency_lower, frequency_upper, monetary_lower,
#' monetary_upper)
#'
#' rfm_create_report(rfm_result, segments, FALSE,
#' "Customer Segmentation Report")
#' }
#'
#' @importFrom utils browseURL
#'
#' @export
#'
rfm_create_report <- function(rfm_table, segments, interactive = FALSE,
                              title = NULL, author = NULL, folder_name = NULL,
                              file_name = NULL) {

  pkg_deps <- c("cli", "DT", "rmarkdown", "knitr", "rmdformats")
  if (interactive) {
    pkg_deps <- append(pkg_deps, "plotly")
  }

  is_installed <- sapply(pkg_deps, try_pkg)
  to_install <- pkg_deps[is.na(is_installed)]

  if (length(to_install) > 0) {
    if (interactive()) {
      message('The following package(s) must be installed:', '\n')
      for (i in seq_len(length(to_install))) {
        cat(paste('-', to_install[i], '\n'))
      }
      message('\n', 'Would you like to install?')
      if (menu(c("Yes", "No")) == 1) {
        install.packages(to_install)
      } else {
        stop('Sorry! The report cannot be generated without installing the
        required packages.', call. = FALSE)
      }
    }
  } else {

    if (is.null(folder_name)) {
      folder_name <- "report"
    }

    if (is.null(file_name)) {
      file_name <- "rfm_report"
    }

    file_rmd <- paste0(folder_name, "/rfm_report.Rmd")
    file_html <- paste0(file_name, ".html")

    template <- system.file("template", "rfm_report.Rmd", package = "rfm")

    cli::cli_bullets(c("i" = paste0("Checking if directory `", folder_name ,
                                    "` exists")))

    if (!dir.exists(folder_name)) {
      cli::cli_bullets(c("v" = paste0("Creating directory `", folder_name ,
                                      "`")))
      dir.create(folder_name)
    } else {
      cli::cli_bullets(c("i" = paste0("Directory `", folder_name ,
                                      "` already exists")))
    }

    cli::cli_bullets(c("i" = "Checking if report template exists"))

    if (!file.exists(file_rmd)) {
      cli::cli_bullets(c("v" = paste0("Copying report template to directory `",
                                      folder_name , "`")))
      file.rename(template, paste0(getwd(), "/", file_rmd))
    } else {
      cli::cli_bullets(c("i" = paste0("Using existing report template in
                                      directory `", folder_name , "`")))
    }

    cli::cli_bullets(c("i" = paste0("Generating `", file_html , "`")))
    rmarkdown::render(file_rmd,
           quiet = TRUE,
           output_dir = folder_name,
           output_file = file_html,
           params = list(data = rfm_table,
                         segments = segments,
                         interactive = interactive,
                         report_title = title,
                         report_author = author))
    cli::cli_bullets(c("v" = "Previewing report"))
    browseURL(paste0(folder_name, "/", file_html))
  }
}

#' RFM table (customer data)
#'
#' Recency, frequency, monetary and RFM score.
#'
#' @param data A \code{data.frame} or \code{tibble}.
#' @param customer_id Unique id of the customer.
#' @param n_transactions Number of transactions/orders.
#' @param recency Days since last visit or date of last visit.
#' @param total_revenue Total revenue from the customer.
#' @param analysis_date Date of analysis.
#' @param recency_bins Number of bins for recency or custom threshold.
#' @param frequency_bins Number of bins for frequency or custom threshold.
#' @param monetary_bins Number of bins for monetary or custom threshold.
#' @param ... Other arguments.
#'
#' @return \code{rfm_table_order} returns a list with the following:
#'
#' \item{rfm}{RFM table.}
#' \item{analysis_date}{Date of analysis.}
#' \item{frequency_bins}{Number of bins used for frequency score.}
#' \item{recency_bins}{Number of bins used for recency score.}
#' \item{monetary_bins}{Number of bins used for monetary score.}
#' \item{threshold}{thresholds used for generating RFM scores.}
#'
#' @examples
#' analysis_date <- as.Date('2007-01-01')
#'
#' # data includes days since last visit
#' rfm_table_customer(rfm_data_customer, customer_id, number_of_orders,
#' recency_days, revenue, analysis_date)
#'
#' # data includes last visit date
#' rfm_table_customer(rfm_data_customer, customer_id, number_of_orders,
#' most_recent_visit, revenue, analysis_date)
#'
#' # access rfm table
#' result <- rfm_table_customer(rfm_data_customer, customer_id, number_of_orders,
#' recency_days, revenue, analysis_date)
#' result$rfm
#'
#' # using custom threshold
#' rfm_table_customer(rfm_data_customer, customer_id, number_of_orders,
#' recency_days, revenue, analysis_date, recency_bins = c(115, 181, 297, 482),
#' frequency_bins = c(4, 5, 6, 8), monetary_bins = c(256, 382, 506, 666))
#'
#' @importFrom dplyr pull
#'
#' @export
#'
rfm_table_customer <- function(data = NULL, customer_id = NULL,
                               n_transactions = NULL, recency = NULL,
                               total_revenue = NULL, analysis_date = NULL,
                               recency_bins = 5, frequency_bins = 5,
                               monetary_bins = 5, ...) UseMethod("rfm_table_customer")

#' @export
#'
rfm_table_customer.default <- function(data = NULL, customer_id = NULL,
                                       n_transactions = NULL, recency = NULL,
                                       total_revenue = NULL, analysis_date = NULL,
                                       recency_bins = 5, frequency_bins = 5,
                                       monetary_bins = 5, ...) {

  col_names <- c("customer_id", "recency_days", "transaction_count", "amount")

  is_days <-
    data %>%
    pull({{ recency }}) %>%
    is.numeric()

  if (is_days) {
    result <-
      data %>%
      select({{ customer_id }}, {{ recency }}, {{ n_transactions }},
             {{ total_revenue }}) %>%
      set_names(col_names)

    other_cols <-
      data %>%
      select(!c({{ recency }}, {{ n_transactions }}, {{ total_revenue }})) %>%
      rename(customer_id = {{ customer_id }})
  } else {
    result <-
      data %>%
      mutate(
        recency_days = as.numeric(analysis_date - {{ recency }},
                                  units = "days"))

    result <-
      result %>%
      select({{ customer_id }}, recency_days, {{ n_transactions }},
             {{ total_revenue }}) %>%
      set_names(col_names)

    other_cols <-
      data %>%
      select(!c(recency_days, {{ n_transactions }}, {{ total_revenue }})) %>%
      rename(customer_id = {{ customer_id }})
  }

  out <- rfm_prep_bins(result, recency_bins, frequency_bins, monetary_bins,
                       analysis_date, other_cols)
  class(out) <- c("rfm_table_customer", "tibble", "data.frame")
  return(out)

}

#' @export
#'
print.rfm_table_customer <- function(x, ...) {
  print(x$rfm)
}

#' @importFrom ggplot2 ggplot geom_tile aes ggtitle xlab ylab scale_fill_gradientn
#' @importFrom RColorBrewer brewer.pal
rfm_gg_heatmap <- function(mapdata, plot_title, xaxis_label, yaxis_label,
                           brewer_n, brewer_name, legend_title) {

  ulm <- ceiling(max(mapdata[["monetary"]]))
  llm <- floor(min(mapdata[["monetary"]]))


  bins <- max(mapdata$frequency_score)
  guide_breaks <- round(seq(llm, ulm, length.out = bins))

  p <-
    ggplot(data = mapdata) +
    geom_tile(aes(x = frequency_score, y = recency_score, fill = monetary)) +
    ggtitle(plot_title) +
    xlab(xaxis_label) +
    ylab(yaxis_label) +
    scale_fill_gradientn(limits = c(llm, ulm),
                        colours = brewer.pal(n = brewer_n, name = brewer_name),
                        name = legend_title)


  return(p)

}

#' @importFrom ggplot2 geom_bar ylim coord_flip geom_text position_dodge
rfm_gg_order_dist <- function(data, flip, bar_color, plot_title, xaxis_label,
                              yaxis_label, ylim_max, bar_labels, bar_label_size) {

  p <-
    data %>%
    ggplot(aes(x = segment, y = n)) +
    geom_bar(stat = "identity", fill = bar_color) +
    ggtitle(plot_title) +
    xlab(xaxis_label) +
    ylab(yaxis_label) +
    ylim(0, ylim_max)

  if (flip) {
    p <-
      p +
      coord_flip()

    if (bar_labels) {
      p <-
        p +
        geom_text(aes(label = sprintf("%1.0f", n), y = n *1.025),
                  hjust = 0,
                  size = bar_label_size)
    }
  } else {
    if (bar_labels) {
      p <-
        p +
        geom_text(aes(label = sprintf("%1.0f", n), y = n * 1.025),
                  position = position_dodge(0.9),
                  vjust = 0,
                  size = bar_label_size)
    }
  }

  return(p)
}

#' @importFrom ggplot2 geom_histogram
rfm_gg_hist <- function(data, hist_bins, hist_color, plot_title, xaxis_label,
                        yaxis_label) {

  p <-
    data %>%
    ggplot(aes(score)) +
    geom_histogram(bins = hist_bins, fill = hist_color, color = I("white")) +
    xlab(xaxis_label) +
    ylab(yaxis_label) +
    ggtitle(plot_title)

  return(p)

}

#' @importFrom ggplot2 theme element_text
#' @importFrom rlang .data
rfm_gg_segment_summary <- function(data, metric, sort, ascending, flip,
                                   bar_color, plot_title, xaxis_label,
                                   yaxis_label, axis_label_size,
                                   axis_label_angle, ylim_max, bar_labels) {

  if (sort) {
    if (ascending) {
      if (flip) {
        p <- ggplot(data,
                    aes(x = reorder(segment, -.data[[metric]], sum),
                        y = .data[[metric]]))
      } else {
        p <- ggplot(data,
                    aes(x = reorder(segment, .data[[metric]], sum),
                        y = .data[[metric]]))
      }
    } else {
      if (flip) {
        p <- ggplot(data,
                    aes(x = reorder(segment, .data[[metric]], sum),
                        y = .data[[metric]]))
      } else {
        p <- ggplot(data,
                    aes(x = reorder(segment, -.data[[metric]], sum),
                        y = .data[[metric]]))
      }
    }
  } else {
    p <- ggplot(data, aes(x = segment, y = .data[[metric]]))
  }

  p <-
    p +
    geom_bar(stat = "identity", fill = bar_color) +
    ggtitle(plot_title) +
    xlab(xaxis_label) +
    ylab(yaxis_label) +
    ylim(0, ylim_max)

  if (flip) {
    p <-
      p +
      coord_flip() +
      theme(axis.text.y = element_text(size = axis_label_size))

    if (bar_labels) {
      p <-
        p +
        geom_text(aes(label = sprintf("%1.0f", .data[[metric]]),
                      y = .data[[metric]] * 1.025),
                  hjust = 0,
                  size = 3)
    }
  } else {
    p <-
      p +
      theme(axis.text.x = element_text(angle = axis_label_angle, vjust = 0,
                                       hjust = 0, size = axis_label_size))

    if (bar_labels) {
      p <-
        p +
        geom_text(aes(label = sprintf("%1.0f", .data[[metric]]),
                      y = .data[[metric]] * 1.025),
                  position = position_dodge(0.9),
                  vjust = 0,
                  size = 3)
    }
  }

  p

}

#' @importFrom ggplot2 scale_fill_manual scale_y_continuous element_blank
#' @importFrom ggplot2 element_line element_rect
rfm_gg_revenue_dist <- function(data, colors, legend_labels, flip,
                                plot_title, xaxis_label, yaxis_label,
                                axis_label_size, axis_label_angle,
                                bar_labels, bar_labels_size) {

  p <-
    ggplot(data, aes(fill = category, y = share, x = segment)) +
    geom_bar(position = "dodge", stat = "identity")


  p <-
    p +
    scale_fill_manual(values = c(colors[1], colors[2]),
                      labels = c(legend_labels[1], legend_labels[2])) +
    scale_y_continuous(labels = scales::percent)

  p <-
    p +
    theme(legend.title = element_blank(),
          legend.position = "bottom",
          panel.background = element_rect(fill = NA),
          axis.ticks = element_line(color = NA))


  if (flip) {
    p <-
      p +
      coord_flip() +
      theme(panel.grid.major.x = element_line(colour = "#ced4da"))

    if (bar_labels) {
      p <-
        p +
        geom_text(aes(label = paste0(round(share * 100, 2), "%"),
                      y = round(share, 2) * 1.03),
                  stat = 'identity', position = position_dodge(0.9),
                  vjust = 0.5, hjust = 0, size = bar_labels_size)
    }
  } else {
    p <-
      p +
      theme(panel.grid.major.y = element_line(colour = "#ced4da"),
            axis.text.x = element_text(angle = axis_label_angle, vjust = 1,
                                       hjust = 0, size = axis_label_size))

    if (bar_labels) {
      p <-
        p +
        geom_text(aes(label = paste0(round(share * 100, 2), "%"),
                      y = round(share, 2) * 1.03),
                  stat = 'identity', position = position_dodge(0.9),
                  vjust = 0, hjust = 0.5, size = bar_labels_size)
    }
  }

  p <-
    p +
    xlab(xaxis_label) +
    ylab(yaxis_label) +
    ggtitle(plot_title)

  return(p)

}

rfm_gg_median <- function(data, bar_color, sort, ascending, flip, plot_title,
                          xaxis_label, yaxis_label, axis_label_size, axis_label_angle,
                          bar_labels) {

  n_fill <- nrow(data)
  cnames <- names(data)
  y_lab  <-
    switch(cnames[2],
           recency_days = "Median Recency",
           transaction_count = "Median Frequency",
           amount = "Median Monetary Value")

  if (is.null(yaxis_label)) {
    yaxis_label <- y_lab
  }

  if (is.null(plot_title)) {
    plot_title <- paste(yaxis_label, " by Segment")
  }

  if (is.null(xaxis_label)) {
    xaxis_label <- "Segment"
  }

  if (is.null(bar_color)) {
    bar_color <- "#0f1a34"
  }

  if (sort) {
    if (ascending) {
      if (flip) {
        p <- ggplot(data,
                    aes(x = reorder(.data[[cnames[1]]], -.data[[cnames[2]]], sum),
                        y = .data[[cnames[2]]]))
      } else {
        p <- ggplot(data,
                    aes(x = reorder(.data[[cnames[1]]], .data[[cnames[2]]], sum),
                        y = .data[[cnames[2]]]))
      }
    } else {
      if (flip) {
        p <- ggplot(data,
                    aes(x = reorder(.data[[cnames[1]]], .data[[cnames[2]]], sum),
                        y = .data[[cnames[2]]]))
      } else {
        p <- ggplot(data,
                    aes(x = reorder(.data[[cnames[1]]], -.data[[cnames[2]]], sum),
                        y = .data[[cnames[2]]]))
      }
    }
  } else {
    p <- ggplot(data, aes(x = .data[[cnames[1]]], y = .data[[cnames[2]]]))
  }


  p <-
    p +
    geom_bar(stat = "identity", fill = bar_color) +
    ggtitle(plot_title) +
    xlab(xaxis_label) +
    ylab(yaxis_label) +
    theme(plot.title = element_text(hjust = 0.5))

  if (flip) {
    p <-
      p +
      coord_flip() +
      theme(axis.text.y = element_text(size = axis_label_size))

    if (bar_labels) {
      p <-
        p +
        geom_text(aes(label = sprintf("%1.0f", .data[[cnames[2]]]),
                      y = .data[[cnames[2]]] * 1.025),
                  hjust = 0,
                  size = 3)
    }
  } else {
    p <-
      p +
      theme(axis.text.x = element_text(angle = axis_label_angle,
                                       size = axis_label_size,
                                       hjust = 0))

    if (bar_labels) {
      p <-
        p +
        geom_text(aes(label = sprintf("%1.0f", .data[[cnames[2]]]),
                      y = .data[[cnames[2]]] * 1.025),
                  position = position_dodge(0.9),
                  vjust = 0,
                  size = 3)
    }
  }

  return(p)

}

#' @importFrom treemapify geom_treemap geom_treemap_text
rfm_gg_segment <- function(table, metric, print_plot) {

  plot <- ggplot(table,
                 aes(area = .data[[metric]],
                     fill = segment,
                     label = paste(toupper(segment),
                                   paste0(.data[[metric]], " (", prop, "%)"),
                                   sep = '\n'))) +
    geom_treemap() +
    geom_treemap_text(size = 8, place = 'centre') +
    theme(legend.position = "none")

  return(plot)

}

rfm_gg_segment_scatter <- function(segments, x_data, y_data, plot_title,
                                   legend_title, xaxis_label, yaxis_label) {

  rfm_gg_combine(segments, x_data, y_data, xaxis_label, yaxis_label,
                 plot_title, legend_title)

}

#' @importFrom ggplot2 geom_point labs
rfm_gg_combine <- function(rfm_table, x = "amount", y = "recency_days",
                           xaxis_title = "Monetary", yaxis_title = "Recency",
                           plot_title = "Recency vs Monetary",
                           legend_title = "Segment") {

  data <- rfm_table[order(rfm_table$rfm_score), ]

  plot <-
    data %>%
    ggplot() +
    geom_point(aes(x = .data[[x]],
                   y = .data[[y]],
                   color = factor(segment),
                   group = seq_along(rfm_score))) +
    xlab(xaxis_title) +
    ylab(yaxis_title) +
    ggtitle(plot_title) +
    labs(color = legend_title)

  return(plot)
}

#' RFM table (transaction data)
#'
#' Recency, frequency, monetary and RFM score.
#'
#' @param data A \code{data.frame} or \code{tibble}.
#' @param customer_id Unique id of the customer.
#' @param order_date Date of the transaction.
#' @param revenue Revenue from the customer.
#' @param analysis_date Date of analysis.
#' @param recency_bins Number of bins for recency or custom threshold.
#' @param frequency_bins Number of bins for frequency or custom threshold.
#' @param monetary_bins Number of bins for monetary or custom threshold.
#' @param ... Other arguments.
#'
#' @return \code{rfm_table_order} returns a list with the following:
#'
#' \item{rfm}{RFM table.}
#' \item{analysis_date}{Date of analysis.}
#' \item{frequency_bins}{Number of bins used for frequency score.}
#' \item{recency_bins}{Number of bins used for recency score.}
#' \item{monetary_bins}{Number of bins used for monetary score.}
#' \item{threshold}{thresholds used for generating RFM scores.}
#'
#' @examples
#' analysis_date <- as.Date("2006-12-31")
#' rfm_table_order(
#'   rfm_data_orders, customer_id, order_date, revenue,
#'   analysis_date
#' )
#'
#' # access rfm table
#' result <- rfm_table_order(
#'   rfm_data_orders, customer_id, order_date,
#'   revenue, analysis_date
#' )
#' result$rfm
#'
#' # using custom threshold
#' rfm_table_order(rfm_data_orders, customer_id, order_date, revenue,
#'   analysis_date,
#'   recency_bins = c(115, 181, 297, 482), frequency_bins = c(4, 5, 6, 8),
#'   monetary_bins = c(256, 382, 506, 666)
#' )
#'
#' @export
#'
rfm_table_order <- function(data = NULL, customer_id = NULL, order_date = NULL,
                            revenue = NULL, analysis_date = NULL,
                            recency_bins = 5, frequency_bins = 5,
                            monetary_bins = 5, ...) UseMethod("rfm_table_order")

#' @importFrom dplyr distinct
#' @export
#'
rfm_table_order.default <- function(data = NULL, customer_id = NULL,
                                    order_date = NULL, revenue = NULL,
                                    analysis_date = NULL, recency_bins = 5,
                                    frequency_bins = 5, monetary_bins = 5, ...) {
  result <- rfm_prep_table_data(
    data, {{ customer_id }}, {{ order_date }},
    {{ revenue }}, analysis_date
  )

  other_cols <-
    data %>%
    select(!c({{ order_date }}, {{ revenue }})) %>%
    distinct() %>%
    rename(customer_id = {{ customer_id }})

  out <- rfm_prep_bins(
    result, recency_bins, frequency_bins, monetary_bins,
    analysis_date, other_cols
  )

  class(out) <- c("rfm_table_order", "tibble", "data.frame")
  return(out)
}

#' @export
#'
print.rfm_table_order <- function(x, ...) {
  print(x$rfm)
}


#' @importFrom dplyr select group_by summarise n mutate
#' @importFrom magrittr %>% %<>% set_names
rfm_prep_table_data <- function(data, customer_id, order_date, revenue,
                                analysis_date) {
  data %>%
    select({{ customer_id }}, {{ order_date }}, {{ revenue }}) %>%
    group_by({{ customer_id }}) %>%
    summarise(
      date_most_recent = max({{ order_date }}),
      amount = sum({{ revenue }}),
      transaction_count = n()
    ) %>%
    mutate(recency_days = as.numeric(analysis_date - date_most_recent,
      units = "days"
    )) %>%
    select(
      {{ customer_id }}, date_most_recent, recency_days, transaction_count,
      amount
    ) %>%
    set_names(c(
      "customer_id", "date_most_recent", "recency_days",
      "transaction_count", "amount"
    ))
}

#' @importFrom rlang int
#' @importFrom dplyr left_join join_by
rfm_prep_bins <- function(result, recency_bins, frequency_bins, monetary_bins,
                          analysis_date, data) {
  result$recency_score <- NA
  result$frequency_score <- NA
  result$monetary_score <- NA

  if (length(recency_bins) == 1) {
    rscore <- rev(seq_len(recency_bins))
    bins_recency <- bins(result, "recency_days", recency_bins)
  } else {
    rscore <- rev(seq_len((length(recency_bins) + 1)))
    bins_recency <- recency_bins
  }

  lower_recency <- bins_lower(result, "recency_days", bins_recency)
  upper_recency <- bins_upper(result, "recency_days", bins_recency)

  rscore_len <- length(rscore)

  for (i in seq_len(rscore_len)) {
    result$recency_score[result$recency_days >= lower_recency[i] &
      result$recency_days < upper_recency[i]] <- rscore[i]
  }


  if (length(frequency_bins) == 1) {
    fscore <- rev(seq_len(frequency_bins))
    bins_frequency <- bins(result, "transaction_count", frequency_bins)
  } else {
    fscore <- rev(seq_len((length(frequency_bins) + 1)))
    bins_frequency <- frequency_bins
  }

  lower_frequency <- bins_lower(result, "transaction_count", bins_frequency)
  upper_frequency <- bins_upper(result, "transaction_count", bins_frequency)

  fscore_len <- length(fscore)

  for (i in seq_len(fscore_len)) {
    result$frequency_score[result$transaction_count >= lower_frequency[i] &
      result$transaction_count < upper_frequency[i]] <- i
  }

  if (length(monetary_bins) == 1) {
    mscore <- rev(seq_len(monetary_bins))
    bins_monetary <- bins(result, "amount", monetary_bins)
  } else {
    mscore <- rev(seq_len((length(monetary_bins) + 1)))
    bins_monetary <- monetary_bins
  }


  lower_monetary <- bins_lower(result, "amount", bins_monetary)
  upper_monetary <- bins_upper(result, "amount", bins_monetary)

  mscore_len <- length(mscore)

  for (i in seq_len(mscore_len)) {
    result$monetary_score[result$amount >= lower_monetary[i] &
      result$amount < upper_monetary[i]] <- i
  }

  result %<>%
    mutate(
      rfm_score = recency_score * 100 + frequency_score * 10 + monetary_score
    ) %>%
    select(
      customer_id, recency_days, transaction_count, amount,
      recency_score, frequency_score, monetary_score, rfm_score
    )

  result$transaction_count <- int(result$transaction_count)

  result <- left_join(result, data, by = join_by(customer_id))

  threshold <- data.frame(
    recency_lower = lower_recency,
    recency_upper = upper_recency,
    frequency_lower = lower_frequency,
    frequency_upper = upper_frequency,
    monetary_lower = lower_monetary,
    monetary_upper = upper_monetary
  )

  list(
    rfm = result,
    analysis_date = analysis_date,
    frequency_bins = frequency_bins,
    recency_bins = recency_bins,
    monetary_bins = monetary_bins,
    threshold = threshold
  )
}

rfm_plotly_heatmap <- function(mapdata, plot_title, xaxis_label, yaxis_label,
                               brewer_n, brewer_name, legend_title) {

  text <- paste0("Frequency Score: ", mapdata$frequency_score,
                 "\nRecency Score: ", mapdata$recency_score,
                 "\nMean Monetary Value: ", round(mapdata$monetary, 2))

  fig <-
    plotly::plot_ly(
      data = mapdata,
      x = ~frequency_score,
      y = ~recency_score,
      z = ~monetary,
      colors = RColorBrewer::brewer.pal(n = brewer_n, name = brewer_name),
      type = "heatmap",
      colorbar = list(title = legend_title),
      hoverinfo = "text",
      hovertext = text
    )

  fig <-
    fig %>%
    plotly::layout(title = paste0("<b>", plot_title, "</b>"),
           xaxis = list(title = paste0("<b>", xaxis_label, "</b>")),
           yaxis = list(title = paste0("<b>", yaxis_label, "</b>")))

  fig %>%
    plotly::config(displayModeBar = FALSE)
}

rfm_plotly_order_dist <- function(data, flip = FALSE, bar_color = NULL,
                                  plot_title = NULL,
                                  xaxis_label = NULL,
                                  yaxis_label = NULL) {

  text <- paste0("Orders: ", data$transaction_count, " \nCustomers: ", data$n)

  if (flip) {
    fig <- plotly::plot_ly(data,
                   x = ~n,
                   y = ~transaction_count,
                   type = "bar",
                   hoverinfo = "text",
                   hovertext = text,
                   orientation = 'h',
                   marker = list(
                     color = bar_color
                   ))
  } else {
    fig <- plotly::plot_ly(data,
                   x = ~transaction_count,
                   y = ~n,
                   type = "bar",
                   hoverinfo = "text",
                   hovertext = text,
                   marker = list(
                     color = bar_color
                   ))
  }

  fig <-
    fig %>%
    plotly::layout(title = paste0("<b>", plot_title, "</b>"),
           xaxis = list(title = paste0("<b>", xaxis_label, "</b>")),
           yaxis = list(title = paste0("<b>", yaxis_label, "</b>")))

  fig %>%
    plotly::config(displayModeBar = FALSE)

}

rfm_plotly_hist <- function(data, hist_color = NULL, plot_title = NULL,
                            xaxis_label = NULL, yaxis_label = NULL) {

  fig <- plotly::plot_ly(data,
                 x = ~score,
                 type = "histogram",
                 histnorm = "count",
                 autobinx = TRUE,
                 marker = list(
                   color = hist_color,
                   line = list(
                     color = "white",
                     width = 1.5
                   )
                 ))


  fig <-
    fig %>%
    plotly::layout(title = paste0("<b>", plot_title, "</b>"),
           xaxis = list(title = paste0("<b>", xaxis_label, "</b>")),
           yaxis = list(title = paste0("<b>", yaxis_label, "</b>")))

  fig %>%
    plotly::config(displayModeBar = FALSE)

}

rfm_plotly_segment_summary <- function(data, metric, flip, sort, ascending,
                                       bar_color, plot_title, xaxis_label,
                                       yaxis_label) {

  text <- paste0("Segment: ", data[["segment"]],
                 "\n", to_title_case(metric), ": ", data[[metric]])

  if (flip) {
    fig <- plotly::plot_ly(data,
                   y = ~segment,
                   x = ~get(metric),
                   type = "bar",
                   hoverinfo = "text",
                   hovertext = text,
                   orientation = 'h',
                   marker = list(
                     color = bar_color
                   ))

    if (sort) {
      if (ascending) {
        fig <- fig %>%
          plotly::layout(title = paste0("<b>", plot_title, "</b>"),
                 xaxis = list(title = paste0("<b>", yaxis_label, "</b>")),
                 yaxis = list(title = paste0("<b>", xaxis_label, "</b>"),
                              categoryorder = "total descending"))
      } else {
        fig <- fig %>%
          plotly::layout(title = paste0("<b>", plot_title, "</b>"),
                 xaxis = list(title = paste0("<b>", yaxis_label, "</b>")),
                 yaxis = list(title = paste0("<b>", xaxis_label, "</b>"),
                              categoryorder = "total ascending"))
      }
    } else {
      fig <- fig %>%
        plotly::layout(title = paste0("<b>", plot_title, "</b>"),
               xaxis = list(title = paste0("<b>", yaxis_label, "</b>")),
               yaxis = list(title = paste0("<b>", xaxis_label, "</b>")))
    }

  } else {
    fig <- plotly::plot_ly(data,
                   x = ~segment,
                   y = ~get(metric),
                   type = "bar",
                   hoverinfo = "text",
                   hovertext = text,
                   marker = list(
                     color = bar_color
                   ))
    if (sort) {
      if (ascending) {
        fig <- fig %>%
          plotly::layout(title = paste0("<b>", plot_title, "</b>"),
                 xaxis = list(title = paste0("<b>", xaxis_label, "</b>"),
                              categoryorder = "total ascending"),
                 yaxis = list(title = paste0("<b>", yaxis_label, "</b>")))
      } else {
        fig <- fig %>%
          plotly::layout(title = paste0("<b>", plot_title, "</b>"),
                 xaxis = list(title = paste0("<b>", xaxis_label, "</b>"),
                              categoryorder = "total descending"),
                 yaxis = list(title = paste0("<b>", yaxis_label, "</b>")))
      }
    } else {
      fig <- fig %>%
        plotly::layout(title = paste0("<b>", plot_title, "</b>"),
               xaxis = list(title = paste0("<b>", xaxis_label, "</b>")),
               yaxis = list(title = paste0("<b>", yaxis_label, "</b>")))
    }
  }

  fig %>%
    plotly::config(displayModeBar = FALSE)

}


rfm_plotly_revenue_dist <- function(x, flip = FALSE,
                                    colors = c("#3b5bdb", "#91a7ff"),
                                    legend_labels = c("Revenue", "Customers"),
                                    plot_title = "Revenue & Customer Distribution",
                                    xaxis_label = NULL, yaxis_label = NULL) {

  x$customer_share <- round((x$customers / sum(x$customers)) * 100, 2)
  x$revenue_share <- round((x$revenue / sum(x$revenue)) * 100, 2)
  data <- x[c("segment", "customer_share", "revenue_share")]

  customer_text <- paste0("Segment: ", data$segment,
                          "\nCustomer Share: ", data$customer_share, "%")

  revenue_text <- paste0("Segment: ", data$segment,
                         "\nRevenue Share: ", data$revenue_share, "%")

  if (is.null(xaxis_label)) {
    xaxis_label <- ""
  }

  if (is.null(yaxis_label)) {
    yaxis_label <- ""
  }

  if (flip) {
    fig <-
    plotly::plot_ly(data,
            x = ~revenue_share,
            y = ~segment,
            type = 'bar',
            orientation = "h",
            name = legend_labels[1],
            hoverinfo = "text",
            hovertext = revenue_text,
            marker = list(color = colors[1])) %>%
    plotly::add_trace(x = ~customer_share,
              name = legend_labels[2],
              hoverinfo = "text",
              hovertext = customer_text,
              marker = list(color = colors[2])) %>%
    plotly::layout(title = paste0("<b>", plot_title, "</b>"),
           xaxis = list(title = paste0("<b>", xaxis_label, "</b>"), ticksuffix = "%"),
           yaxis = list(title = paste0("<b>", yaxis_label, "</b>")),
           legend = list(x = 100, y = 0.5))
  } else {
    fig <-
    plotly::plot_ly(data,
            x = ~segment,
            y = ~revenue_share,
            type = 'bar',
            name = legend_labels[1],
            hoverinfo = "text",
            hovertext = revenue_text,
            marker = list(color = colors[1])) %>%
    plotly::add_trace(y = ~customer_share,
              name = legend_labels[2],
              hoverinfo = "text",
              hovertext = customer_text,
              marker = list(color = colors[2])) %>%
    plotly::layout(title = paste0("<b>", plot_title, "</b>"),
           xaxis = list(title = paste0("<b>", xaxis_label, "</b>")),
           yaxis = list(title = paste0("<b>", yaxis_label, "</b>"), ticksuffix = "%"),
           legend = list(x = 100, y = 0.5))
  }

  fig <-
    fig %>%
    plotly::layout(title = paste0("<b>", plot_title, "</b>"),
           xaxis = list(title = paste0("<b>", xaxis_label, "</b>")),
           yaxis = list(title = paste0("<b>", yaxis_label, "</b>"), ticksuffix = "%"),
           legend = list(x = 100, y = 0.5))

  fig %>%
    plotly::config(displayModeBar = FALSE)

}


rfm_plotly_segment <- function(table, metric = "customers") {

  text <- paste0("Segment:", toupper(table$segment), "\n",
                 to_title_case(metric), ": ", table[[metric]],
                 " (", table$prop, "%)")

  fig <- plotly::plot_ly(
    type = "treemap",
    labels = table$segment,
    parents = rep("", nrow(table)),
    values = table[[metric]],
    hoverinfo = "text",
    hovertext = text
  )

  fig %>%
    plotly::config(displayModeBar = FALSE)
}

rfm_plotly_segment_scatter <- function(segments, x_data = NULL, y_data = NULL,
                                       plot_title = NULL, legend_title = NULL,
                                       xaxis_label = NULL, yaxis_label = NULL) {

  if (is.null(legend_title)) {
    legend_title <- paste0('<b> ', legend_title , ' </b>')
  } else {
    legend_title <- legend_title
  }

  text <- paste0("Amount: ", segments[[x_data]], "\nRecency: ",
                 segments[[y_data]], "\nSegment: ", segments$segment)



  fig <- plotly::plot_ly(segments,
                 x = ~get(x_data),
                 y = ~get(y_data),
                 type = "scatter",
                 mode = "markers",
                 color = ~factor(segment),
                 colors = "Paired",
                 hoverinfo = "text",
                 hovertext = text)

  fig <-
    fig %>%
    plotly::layout(title = paste0("<b>", plot_title, "</b>"),
           xaxis = list(title = paste0("<b>", xaxis_label, "</b>")),
           yaxis = list(title = paste0("<b>", yaxis_label, "</b>")),
           legend = list(title = list(text = legend_title)))

  fig %>%
    plotly::config(displayModeBar = FALSE)

}

rfm_plotly_median <- function(data, bar_color = NULL, sort = FALSE,
                              ascending = FALSE, flip = FALSE,
                              plot_title = NULL, xaxis_label = NULL,
                              yaxis_label = NULL) {

  cnames <- colnames(data)

  ycol <- switch(cnames[2],
                 recency_days   = "Median Recency",
                 transaction_count = "Median Frequency",
                 amount  = "Median Monetary Value"
  )

  text <- paste0("Segment: ", data[[cnames[1]]],
                 " \nMedian Recency: ", data[[cnames[2]]])

  if (is.null(yaxis_label)) {
    yaxis_label <- ycol
  }

  if (is.null(plot_title)) {
    plot_title <- paste(yaxis_label, " by Segment")
  }

  if (is.null(xaxis_label)) {
    xaxis_label <- "Segment"
  }

  if (is.null(bar_color)) {
    bar_color <- "#0f1a34"
  }

  if (flip) {
    fig <- plotly::plot_ly(data,
                   y = ~segment,
                   x = ~get(cnames[2]),
                   type = "bar",
                   hoverinfo = "text",
                   hovertext = text,
                   orientation = 'h',
                   marker = list(
                     color = bar_color
                   ))

    if (sort) {
      if (ascending) {
        fig <- fig %>%
          plotly::layout(title = paste0("<b>", plot_title, "</b>"),
                 xaxis = list(title = paste0("<b>", yaxis_label, "</b>")),
                 yaxis = list(title = paste0("<b>", xaxis_label, "</b>"),
                              categoryorder = "total descending"))
      } else {
        fig <- fig %>%
          plotly::layout(title = paste0("<b>", plot_title, "</b>"),
                 xaxis = list(title = paste0("<b>", yaxis_label, "</b>")),
                 yaxis = list(title = paste0("<b>", xaxis_label, "</b>"),
                              categoryorder = "total ascending"))
      }
    } else {
      fig <- fig %>%
        plotly::layout(title = paste0("<b>", plot_title, "</b>"),
               xaxis = list(title = paste0("<b>", yaxis_label, "</b>")),
               yaxis = list(title = paste0("<b>", xaxis_label, "</b>")))
    }

  } else {
    fig <- plotly::plot_ly(data,
                   x = ~segment,
                   y = ~get(cnames[2]),
                   type = "bar",
                   hoverinfo = "text",
                   hovertext = text,
                   marker = list(
                     color = bar_color
                   ))
    if (sort) {
      if (ascending) {
        fig <- fig %>%
          plotly::layout(title = paste0("<b>", plot_title, "</b>"),
                 xaxis = list(title = paste0("<b>", xaxis_label, "</b>"),
                              categoryorder = "total ascending"),
                 yaxis = list(title = paste0("<b>", yaxis_label, "</b>")))
      } else {
        fig <- fig %>%
          plotly::layout(title = paste0("<b>", plot_title, "</b>"),
                 xaxis = list(title = paste0("<b>", xaxis_label, "</b>"),
                              categoryorder = "total descending"),
                 yaxis = list(title = paste0("<b>", yaxis_label, "</b>")))
      }
    } else {
      fig <- fig %>%
        plotly::layout(title = paste0("<b>", plot_title, "</b>"),
               xaxis = list(title = paste0("<b>", xaxis_label, "</b>")),
               yaxis = list(title = paste0("<b>", yaxis_label, "</b>")))
    }
  }

  fig %>%
    plotly::config(displayModeBar = FALSE)
}

#' @title Launch shiny app
#' @description Launches shiny app.
#' @examples
#' \dontrun{
#' rfm_launch_app()
#' }
#' @export
#'
rfm_launch_app <- function() {

	pkg_deps <- c('haven', 'readr', 'readxl', 'jsonlite',
              'shinyBS', 'shinycssloaders', 'shinythemes',
              'stringr', 'DT')

	is_installed <- sapply(pkg_deps, try_pkg)
	to_install <- pkg_deps[is.na(is_installed)]

	if (length(to_install) > 0) {
		if (interactive()) {
			message('The following package(s) must be installed:', '\n')
			for (i in seq_len(length(to_install))) {
				cat(paste('-', to_install[i], '\n'))
			}
			message('\n', 'Would you like to install?')
			if (menu(c("Yes", "No")) == 1) {
				install.packages(to_install)
				xplorerr::app_rfm_analysis()
			} else {
				stop('Sorry! The app cannot be launched without installing the required
				     packages.', call. = FALSE)
			}
		}
	} else {
		xplorerr::app_rfm_analysis()
	}


}

---
title: "`r params$report_title`"
author: "`r params$report_author`"
date: "`r Sys.Date()`"
output:
  rmdformats::html_clean:
    highlight: kate
    fig_width: 8
    fig_height: 6
    thumbnails: FALSE
params:
  data: NULL
  segments: NULL
  interactive: FALSE
  report_title: Customer Segmentation Report
  report_author: NULL
---

```{css, echo=FALSE}
#toc {
  width: 20%;
  padding: 20px 1px 5px 1px;
}
```

```{r setup, include=FALSE}
## Global options
library(rfm)
library(dplyr)
library(magrittr)
library(DT)
knitr::opts_chunk$set(echo = FALSE, width = 75)
```

# RFM Score

```{r rfm_score}
data <- params$data
data %>% 
  use_series(rfm) %>% 
  select(customer_id, recency_days, transaction_count, amount, rfm_score) %>% 
  datatable()
```

# EDA

## Heatmap

```{r heatmap, fig.align='center'}
rfm_plot_heatmap(data, interactive = params$interactive)
```

## Bar Chart

```{r bar_chart, fig.align='center'}
rfm_plot_bar_chart(data)
```

## Order Distribution

```{r order_dist, fig.align='center'}
rfm_plot_order_dist(data, interactive = params$interactive)
```

## Recency Distribtuion

```{r hist_recency, fig.align='center'}
rfm_plot_histogram(data, interactive = params$interactive)
```

## Frequency Distribtuion

```{r hist_frequency, fig.align='center'}
rfm_plot_histogram(data, "frequency", interactive = params$interactive)
```

## Monetary Distribtuion

```{r hist_monetary, fig.align='center'}
rfm_plot_histogram(data, "monetary", interactive = params$interactive)
```

# Segmentation

## Segments

```{r segments}
segments <- params$segments
segments %>% 
  select(customer_id, segment, rfm_score, transaction_count, recency_days,
         amount) %>% 
  datatable()
```

## Segment Summary

```{r segment_summary}
segment_overview <- rfm_segment_summary(segments)
segment_overview %>%
  datatable()
```

## Segment Summary Plot

```{r plot_segment_summary, fig.align='center'}
rfm_plot_segment_summary(segment_overview, interactive = params$interactive)
```

## Segmentation Plot

```{r plot_segment, fig.align='center'}
rfm_plot_segment(segment_overview, interactive = params$interactive)
```

## Revenue Distribution

```{r plot_revenue_dist, fig.align='center'}
rfm_plot_revenue_dist(segment_overview, interactive = params$interactive)  
```

## Median Recency

```{r plot_median_recency, fig.align='center'}
rfm_plot_median_recency(segments, interactive = params$interactive)
```

## Median Frequency

```{r plot_median_frequency, fig.align='center'}
rfm_plot_median_frequency(segments, interactive = params$interactive)
```

## Median Monetary Value

```{r plot_median_monetary, fig.align='center'}
rfm_plot_median_monetary(segments, interactive = params$interactive)
```

## Recency vs Monetary Value

```{r plot_recency_monetary, fig.align='center'}
rfm_plot_segment_scatter(segments, "monetary", "recency", interactive = params$interactive)
```

## Recency vs Frequency

```{r plot_recency_frequency, fig.align='center'}
rfm_plot_segment_scatter(segments, "frequency", "recency", interactive = params$interactive)
```

## Frequency vs Monetary Value

```{r plot_frequency_monetary, fig.align='center'}
rfm_plot_segment_scatter(segments, "monetary", "frequency", interactive = params$interactive)
```

---
title: "RFM - Introduction"
author: "Aravind Hebbali"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{RFM - Introduction} 
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

## Introduction

```{r, echo=FALSE, message=FALSE}
library(rfm)
library(dplyr)
library(rlang)
library(scales)
library(knitr)
library(kableExtra)
library(magrittr)
library(ggplot2)
library(DT)
library(grDevices)
library(RColorBrewer)
options(knitr.table.format = "html")
```

This article provides a brief introduction to RFM analysis and customer segmentation. If you are looking for a detailed guide, check out our free [online course](https://rsquared-academy.thinkific.com/courses/customer-segmentation-using-rfm-analysis) or YouTube [tutorial](https://www.youtube.com/watch?v=275X7yaSsoQ) or blog [post](https://blog.rsquaredacademy.com/customer-segmentation-using-rfm-analysis/).

**RFM** (recency, frequency, monetary) analysis is a behavior based technique used to segment customers by examining their transaction history  such as

- how recently a customer has purchased (recency)
- how often they purchase (frequency)
- how much the customer spends (monetary)

It is based on the marketing axiom that **80% of your business comes from 20% of your customers**. RFM helps to identify customers who are more likely to respond to promotions by segmenting them into various categories.

## Data

To calculate the RFM score for each customer we need data for a particular time frame and should include the following:

- a unique customer id 
- date of transaction/order
- transaction/order amount

Data can be at customer level or transaction level i.e. each row in the data may represent a single transaction of a customer or summary of all transactions of a customer. `rfm` package includes two sample data sets:

- `rfm_data_orders`

```{r rfm_data_orders}
head(rfm_data_orders)
```

- `rfm_data_customer`

```{r rfm_data_customer}
head(rfm_data_customer)
```

You can take a look at them to understand the difference between customer and transaction level data. **Remember, the data sets are different and the final results will not match.**

## RFM Score

So how is the RFM score computed for each customer? The below steps explain the process:

- A recency score is assigned to each customer based on date of most recent purchase. The score is generated by binning the recency values into a number of categories (default is 5). For example, if you use four categories, the customers with the most recent purchase dates receive a recency ranking of 4, and those with purchase dates in the distant past receive a recency ranking of 1.

- A frequency ranking is assigned in a similar way. Customers with high purchase frequency are assigned a higher score (4 or 5) and those with lowest frequency are assigned a score 1.

- Monetary score is assigned on the basis of the total revenue generated by the customer in the period under consideration for the analysis. Customers with highest revenue/order amount are assigned a higher score while those with lowest revenue are assigned a score of 1.

- A fourth score, RFM score is generated which is simply the three individual scores concatenated into a single value.

The customers with the highest RFM scores are most likely to respond to an offer. Now that we have understood how the RFM score is computed, it is time to put it into practice. Use `rfm_table_order()` to generate the score for each customer from the sample data set `rfm_data_orders`.

```{r rfm_table_order, eval=FALSE}
analysis_date <- as.Date("2006-12-31")
rfm_result <- rfm_table_order(rfm_data_orders, customer_id, order_date, revenue, analysis_date)
rfm_result
```

```{r rfm_table_order2, eval=TRUE, echo=FALSE}
analysis_date <- as.Date("2006-12-31")
rfm_result <- rfm_table_order(rfm_data_orders, customer_id, order_date, revenue, analysis_date)
rfm_result$rfm[1:10, ] %>%
  select(customer_id, recency_days, transaction_count, amount, rfm_score, recency_score, frequency_score, monetary_score, first_name, last_name, email) %>% 
  kable() %>%
  kable_styling()
```

## Segments

Let us segment our customers based on the individual recency, frequency and monetary scores. Keep in mind that creating segments based on RFM score is a very subjective endeavour. Having good business and domain knowledge will allow the user to generate effective segments. There is no one size fits all solution here.

```{r segments, echo=FALSE}
segment <- c("Champions", "Potential Loyalist", "Loyal Customers", 
             "Promising", "New Customers", "Can't Lose Them", 
             "At Risk", "Need Attention", "About To Sleep", "Lost")
description <- c(
  "Bought recently, buy often and spend the most",
  "Recent customers, spent good amount, bought more than once",
  "Spend good money. Responsive to promotions",
  "Recent shoppers, but haven't spent much",
  "Bought more recently, but not often",
  "Made big purchases and often, but long time ago",
  "Spent big money, purchased often but long time ago",
  "Above average recency, frequency & monetary values",
  "Below average recency, frequency & monetary values",
  "Bought a long time ago, average amount spent"
)
recency <-   c("5", "3 - 5", "2 - 4", "3 - 4", "4 - 5", "1 - 2", "1 - 2", "1 - 3", "2 - 3", "1 - 1")
frequency <- c("5", "3 - 5", "2 - 4", "1 - 3", "1 - 3", "3 - 4", "2 - 5", "3 - 5", "1 - 3", "1 - 5")
monetary <-  c("5", "2 - 5", "2 - 4", "3 - 5", "1 - 5", "4 - 5", "4 - 5", "3 - 5", "1 - 4", "1 - 5")
segments <- data.frame(
  Segment = segment, Description = description,
  R = recency, `F` = frequency, M = monetary
)

segments %>%
  kable() %>%
  kable_styling(full_width = FALSE, font_size = 12)
```

## Segmented Customer Data

We can use the segmented data to identify 

- best customers
- loyal customers
- at risk customers
- and lost customers

Once we have segmented a customer, we can take appropriate action to increase his/her lifetime value. 

```{r criteria, echo=FALSE}
segment_names <- c("Champions", "Potential Loyalist", "Loyal Customers", 
                   "Promising", "New Customers", "Can't Lose Them", 
                   "At Risk", "Need Attention", "About To Sleep", "Lost")

recency_lower <-   c(5, 3, 2, 3, 4, 1, 1, 1, 2, 1)
recency_upper <-   c(5, 5, 4, 4, 5, 2, 2, 3, 3, 1)
frequency_lower <- c(5, 3, 2, 1, 1, 3, 2, 3, 1, 1)
frequency_upper <- c(5, 5, 4, 3, 3, 4, 5, 5, 3, 5)
monetary_lower <-  c(5, 2, 2, 3, 1, 4, 4, 3, 1, 1)
monetary_upper <-  c(5, 5, 4, 5, 5, 5, 5, 5, 4, 5)

segments <- rfm_segment(rfm_result, segment_names, recency_lower, recency_upper,
frequency_lower, frequency_upper, monetary_lower, monetary_upper)

segments %>%
  head(10) %>%
  kable() %>%
  kable_styling()
```

Let us quickly summarize the segments to get an overview of the number of customers, orders and average order value in each of them.

#### Segment Summary

```{r segment_summary}
rfm_segment_summary(segments)
```

`rfm` package offers visualization tools to validate the segments generated from the RFM score. Below are a few of them:

#### Segmentation Plot

```{r segment_plot, fig.align='center', fig.width=7, fig.height=7}
segments %>%
  rfm_segment_summary() %>%
  rfm_plot_segment()
```

#### Segment Summary Plot

```{r segment_sumamry_plot, fig.align='center', fig.width=7, fig.height=7}
segments %>%
  rfm_segment_summary() %>%
  rfm_plot_segment_summary()
```

#### Revenue Distribution

```{r revenue_dist_plot, fig.align='center', fig.width=7, fig.height=7}
segments %>%
  rfm_segment_summary() %>%
  rfm_plot_revenue_dist()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rfm-plots.R
\name{rfm_plot_heatmap}
\alias{rfm_plot_heatmap}
\alias{rfm_heatmap}
\title{RFM heatmap}
\usage{
rfm_plot_heatmap(
  data,
  brewer_n = 5,
  brewer_name = "PuBu",
  xaxis_label = NULL,
  yaxis_label = NULL,
  plot_title = NULL,
  legend_title = NULL,
  interactive = FALSE,
  print_plot = TRUE
)
}
\arguments{
\item{data}{An object of class \code{rfm_table}.}

\item{brewer_n}{Indicates the number of colors in the palette; RColorBrewer
is used for the color palette of the heatmap; check the documentation of
\code{brewer.pal}.}

\item{brewer_name}{Palette name; check the documentation of
\code{brewer.pal}.}

\item{xaxis_label}{X axis label.}

\item{yaxis_label}{Y axis label.}

\item{plot_title}{Title of the plot.}

\item{legend_title}{Legend title.}

\item{interactive}{If \code{TRUE}, uses \code{plotly} as the visualization
engine. If \code{FALSE}, uses \code{ggplot2}.}

\item{print_plot}{logical; if \code{TRUE}, prints the plot else returns a
plot object.}
}
\description{
The heat map shows the average monetary value for different
  categories of recency and frequency scores. Higher scores of frequency and
  recency are characterized by higher average monetary value as indicated by
  the darker areas in the heatmap.
}
\section{Deprecated Functions}{

\code{rfm_heatmap()} has been deprecated and will be made defunct. It has
been provided for compatibility with older versions only, and will be made
defunct at the next release.

Instead use the replacement function \code{rfm_plot_heatmap()}.
}

\examples{
# using transaction data
analysis_date <- as.Date('2006-12-31')
rfm_order <- rfm_table_order(rfm_data_orders, customer_id, order_date,
revenue, analysis_date)

# heat map
# ggplot2
rfm_plot_heatmap(rfm_order)

# plotly
rfm_plot_heatmap(rfm_order, interactive = TRUE)

# using customer data
analysis_date <- as.Date('2007-01-01')
rfm_customer <- rfm_table_customer(rfm_data_customer, customer_id,
number_of_orders, recency_days, revenue, analysis_date)

# heat map
rfm_plot_heatmap(rfm_customer)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rfm-plots.R
\name{rfm_plot_bar_chart}
\alias{rfm_plot_bar_chart}
\alias{rfm_bar_chart}
\title{RFM bar chart}
\usage{
rfm_plot_bar_chart(
  rfm_table,
  bar_color = NULL,
  xaxis_label = NULL,
  sec_xaxis_label = NULL,
  yaxis_label = NULL,
  sec_yaxis_label = NULL,
  print_plot = TRUE
)
}
\arguments{
\item{rfm_table}{An object of class \code{rfm_table}.}

\item{bar_color}{Color of the bars.}

\item{xaxis_label}{X axis label.}

\item{sec_xaxis_label}{Secondary x axis label.}

\item{yaxis_label}{Y axis label.}

\item{sec_yaxis_label}{Secondary y axis label.}

\item{print_plot}{logical; if \code{TRUE}, prints the plot else returns a
plot object.}
}
\value{
Bar chart.
}
\description{
Examine the distribution of monetary scores for the different
  combinations of frequency and recency scores.
}
\section{Deprecated Functions}{

\code{rfm_bar_chart()} has been deprecated and will be made defunct. It has
been provided for compatibility with older versions only, and will be made
defunct at the next release.

Instead use the replacement function \code{rfm_plot_bar_chart()}.
}

\examples{
# using transaction data
analysis_date <- as.Date('2006-12-31')
rfm_order <- rfm_table_order(rfm_data_orders, customer_id, order_date,
revenue, analysis_date)

# bar chart
rfm_plot_bar_chart(rfm_order)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rfm-plots.R
\name{rfm_rm_plot}
\alias{rfm_rm_plot}
\alias{rfm_fm_plot}
\alias{rfm_rf_plot}
\title{RFM Scatter plot}
\usage{
rfm_rm_plot(
  segments,
  xaxis_label = NULL,
  yaxis_label = NULL,
  plot_title = NULL,
  print_plot = TRUE
)

rfm_fm_plot(
  segments,
  xaxis_label = NULL,
  yaxis_label = NULL,
  plot_title = NULL,
  print_plot = TRUE
)

rfm_rf_plot(
  segments,
  xaxis_label = NULL,
  yaxis_label = NULL,
  plot_title = NULL,
  print_plot = TRUE
)
}
\arguments{
\item{segments}{Output from \code{rfm_segment}.}

\item{xaxis_label}{X axis label.}

\item{yaxis_label}{Y axis label.}

\item{plot_title}{Title of the plot.}

\item{print_plot}{logical; if \code{TRUE}, prints the plot else returns a
plot object.}
}
\value{
Scatter plot.
}
\description{
Examine the relationship between recency, frequency
  and monetary values.
}
\section{Deprecated Functions}{

\code{rfm_rm_plot()}, \code{rfm_fm_plot()} and \code{rfm_rf_plot()} have
been deprecated and will be made defunct. These functions have been provided
for compatibility with older versions only, and will be made defunct at the
next release.
Instead use the replacement function \code{rfm_plot_segment_scatter()}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rfm-table-transaction.R
\name{rfm_table_order}
\alias{rfm_table_order}
\title{RFM table (transaction data)}
\usage{
rfm_table_order(
  data = NULL,
  customer_id = NULL,
  order_date = NULL,
  revenue = NULL,
  analysis_date = NULL,
  recency_bins = 5,
  frequency_bins = 5,
  monetary_bins = 5,
  ...
)
}
\arguments{
\item{data}{A \code{data.frame} or \code{tibble}.}

\item{customer_id}{Unique id of the customer.}

\item{order_date}{Date of the transaction.}

\item{revenue}{Revenue from the customer.}

\item{analysis_date}{Date of analysis.}

\item{recency_bins}{Number of bins for recency or custom threshold.}

\item{frequency_bins}{Number of bins for frequency or custom threshold.}

\item{monetary_bins}{Number of bins for monetary or custom threshold.}

\item{...}{Other arguments.}
}
\value{
\code{rfm_table_order} returns a list with the following:

\item{rfm}{RFM table.}
\item{analysis_date}{Date of analysis.}
\item{frequency_bins}{Number of bins used for frequency score.}
\item{recency_bins}{Number of bins used for recency score.}
\item{monetary_bins}{Number of bins used for monetary score.}
\item{threshold}{thresholds used for generating RFM scores.}
}
\description{
Recency, frequency, monetary and RFM score.
}
\examples{
analysis_date <- as.Date("2006-12-31")
rfm_table_order(
  rfm_data_orders, customer_id, order_date, revenue,
  analysis_date
)

# access rfm table
result <- rfm_table_order(
  rfm_data_orders, customer_id, order_date,
  revenue, analysis_date
)
result$rfm

# using custom threshold
rfm_table_order(rfm_data_orders, customer_id, order_date, revenue,
  analysis_date,
  recency_bins = c(115, 181, 297, 482), frequency_bins = c(4, 5, 6, 8),
  monetary_bins = c(256, 382, 506, 666)
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rfm-shiny-app.R
\name{rfm_launch_app}
\alias{rfm_launch_app}
\title{Launch shiny app}
\usage{
rfm_launch_app()
}
\description{
Launches shiny app.
}
\examples{
\dontrun{
rfm_launch_app()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rfm-segment.R
\name{rfm_plot_median_recency}
\alias{rfm_plot_median_recency}
\alias{rfm_plot_median_frequency}
\alias{rfm_plot_median_monetary}
\title{Median plots}
\usage{
rfm_plot_median_recency(
  rfm_segment_table,
  sort = FALSE,
  ascending = FALSE,
  flip = FALSE,
  bar_color = NULL,
  plot_title = NULL,
  xaxis_label = NULL,
  yaxis_label = NULL,
  axis_label_size = 8,
  axis_label_angle = 315,
  bar_labels = TRUE,
  interactive = FALSE,
  animate = FALSE,
  print_plot = TRUE
)

rfm_plot_median_frequency(
  rfm_segment_table,
  sort = FALSE,
  ascending = FALSE,
  flip = FALSE,
  bar_color = NULL,
  plot_title = NULL,
  xaxis_label = NULL,
  yaxis_label = NULL,
  axis_label_size = 8,
  axis_label_angle = 315,
  bar_labels = TRUE,
  interactive = FALSE,
  animate = FALSE,
  print_plot = TRUE
)

rfm_plot_median_monetary(
  rfm_segment_table,
  sort = FALSE,
  ascending = FALSE,
  flip = FALSE,
  bar_color = NULL,
  plot_title = NULL,
  xaxis_label = NULL,
  yaxis_label = NULL,
  axis_label_size = 8,
  axis_label_angle = 315,
  bar_labels = TRUE,
  interactive = FALSE,
  animate = FALSE,
  print_plot = TRUE
)
}
\arguments{
\item{rfm_segment_table}{Output from \code{rfm_segment}.}

\item{sort}{logical; if \code{TRUE}, sort metrics.}

\item{ascending}{logical; if \code{TRUE}, sort metrics in ascending order.}

\item{flip}{logical; if \code{TRUE}, creates horizontal bar plot.}

\item{bar_color}{Color of the bars.}

\item{plot_title}{Title of the plot.}

\item{xaxis_label}{X axis label.}

\item{yaxis_label}{Y axis label.}

\item{axis_label_size}{Font size of X axis tick labels.}

\item{axis_label_angle}{Angle of X axis tick labels.}

\item{bar_labels}{If \code{TRUE}, add labels to the bars. Defaults to
\code{TRUE}.}

\item{interactive}{If \code{TRUE}, uses \code{plotly} as the visualization
engine. If \code{FALSE}, uses \code{ggplot2}.}

\item{animate}{If \code{TRUE}, animates the bars. Defaults to \code{FALSE}.}

\item{print_plot}{logical; if \code{TRUE}, prints the plot else returns a
plot object.}
}
\description{
Segment wise median recency, frequency & monetary value plot.
}
\examples{
# analysis date
analysis_date <- as.Date('2006-12-31')

# generate rfm score
rfm_result <- rfm_table_order(rfm_data_orders, customer_id, order_date,
revenue, analysis_date)

# segment names
segment_names <- c("Champions", "Potential Loyalist", "Loyal Customers",
                   "Promising", "New Customers", "Can't Lose Them",
                   "At Risk", "Need Attention", "About To Sleep", "Lost")

# segment intervals
recency_lower <-   c(5, 3, 2, 3, 4, 1, 1, 1, 2, 1)
recency_upper <-   c(5, 5, 4, 4, 5, 2, 2, 3, 3, 1)
frequency_lower <- c(5, 3, 2, 1, 1, 3, 2, 3, 1, 1)
frequency_upper <- c(5, 5, 4, 3, 3, 4, 5, 5, 3, 5)
monetary_lower <-  c(5, 2, 2, 3, 1, 4, 4, 3, 1, 1)
monetary_upper <-  c(5, 5, 4, 5, 5, 5, 5, 5, 4, 5)

# generate segments
segments <- rfm_segment(rfm_result, segment_names, recency_lower,
recency_upper, frequency_lower, frequency_upper, monetary_lower,
monetary_upper)

# plots
# visualize median recency
rfm_plot_median_recency(segments)

# plotly
rfm_plot_median_recency(segments, interactive = TRUE)

# sort in ascending order
rfm_plot_median_recency(segments, sort = TRUE, ascending = TRUE)

# default sorting is in descending order
rfm_plot_median_recency(segments, sort = TRUE)

# horizontal bars
rfm_plot_median_recency(segments, flip = TRUE)

# median frequency
rfm_plot_median_frequency(segments)

# median monetary value
rfm_plot_median_monetary(segments)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rfm-report.R
\name{rfm_create_report}
\alias{rfm_create_report}
\title{RFM report}
\usage{
rfm_create_report(
  rfm_table,
  segments,
  interactive = FALSE,
  title = NULL,
  author = NULL,
  folder_name = NULL,
  file_name = NULL
)
}
\arguments{
\item{rfm_table}{An object of class \code{rfm_table}.}

\item{segments}{Output from \code{rfm_segment}.}

\item{interactive}{If \code{TRUE}, uses \code{plotly} as the visualization
engine. If \code{FALSE}, uses \code{ggplot2}.}

\item{title}{Title of the report.}

\item{author}{Author of the report.}

\item{folder_name}{The output directory for the report.}

\item{file_name}{The name of the report file.}
}
\description{
Generates a segmentation analysis report.
}
\examples{
\dontrun{
# analysis date
analysis_date <- as.Date('2006-12-31')

# generate rfm score
rfm_result <- rfm_table_order(rfm_data_orders, customer_id, order_date,
revenue, analysis_date)

# segment names
segment_names <- c("Champions", "Potential Loyalist", "Loyal Customers",
                   "Promising", "New Customers", "Can't Lose Them",
                   "At Risk", "Need Attention", "About To Sleep", "Lost")

# segment intervals
recency_lower <-   c(5, 3, 2, 3, 4, 1, 1, 1, 2, 1)
recency_upper <-   c(5, 5, 4, 4, 5, 2, 2, 3, 3, 1)
frequency_lower <- c(5, 3, 2, 1, 1, 3, 2, 3, 1, 1)
frequency_upper <- c(5, 5, 4, 3, 3, 4, 5, 5, 3, 5)
monetary_lower <-  c(5, 2, 2, 3, 1, 4, 4, 3, 1, 1)
monetary_upper <-  c(5, 5, 4, 5, 5, 5, 5, 5, 4, 5)

# generate segments
segments <- rfm_segment(rfm_result, segment_names, recency_lower,
recency_upper, frequency_lower, frequency_upper, monetary_lower,
monetary_upper)

rfm_create_report(rfm_result, segments, FALSE,
"Customer Segmentation Report")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rfm-plots-data.R
\name{rfm_barchart_data}
\alias{rfm_barchart_data}
\title{Bar chart data}
\usage{
rfm_barchart_data(rfm_table)
}
\arguments{
\item{rfm_table}{An object of class \code{rfm_table}.}
}
\description{
Data for generating bar charts.
}
\examples{
# using transaction data
analysis_date <- as.Date('2006-12-31')
rfm_order <- rfm_table_order(rfm_data_orders, customer_id, order_date,
revenue, analysis_date)

# bar chart data
rfm_barchart_data(rfm_order)

# using customer data
analysis_date <- as.Date('2007-01-01')
rfm_customer <- rfm_table_customer(rfm_data_customer, customer_id,
number_of_orders, recency_days, revenue, analysis_date)

# bar chart data
rfm_barchart_data(rfm_customer)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rfm-plots.R
\name{rfm_plot_order_dist}
\alias{rfm_plot_order_dist}
\alias{rfm_order_dist}
\title{Customers by orders}
\usage{
rfm_plot_order_dist(
  rfm_table,
  flip = FALSE,
  bar_color = NULL,
  plot_title = NULL,
  xaxis_label = NULL,
  yaxis_label = NULL,
  bar_label_size = 3,
  bar_labels = TRUE,
  interactive = FALSE,
  animate = FALSE,
  print_plot = TRUE
)
}
\arguments{
\item{rfm_table}{An object of class \code{rfm_table}.}

\item{flip}{logical; if \code{TRUE}, creates horizontal bar plot.}

\item{bar_color}{Color of the bars.}

\item{plot_title}{Title of the plot.}

\item{xaxis_label}{X axis title.}

\item{yaxis_label}{Y axis title.}

\item{bar_label_size}{Size of bar labels.}

\item{bar_labels}{If \code{TRUE}, add labels to the bars. Defaults to
\code{TRUE}.}

\item{interactive}{If \code{TRUE}, uses \code{plotly} as the visualization
engine. If \code{FALSE}, uses \code{ggplot2}.}

\item{animate}{If \code{TRUE}, animates the bars. Defaults to \code{FALSE}.}

\item{print_plot}{logical; if \code{TRUE}, prints the plot else returns a
plot object.}
}
\value{
Bar chart.
}
\description{
Visualize the distribution of customers across orders.
}
\section{Deprecated Functions}{

\code{rfm_order_dist()} has been deprecated and will be made defunct. It has
been provided for compatibility with older versions only, and will be made
defunct at the next release.

Instead use the replacement function \code{rfm_plot_order_dist()}.
}

\examples{
# using transaction data
analysis_date <- as.Date('2006-12-31')
rfm_order <- rfm_table_order(rfm_data_orders, customer_id, order_date,
revenue, analysis_date)

# order distribution
rfm_plot_order_dist(rfm_order)

# horizontal bars
rfm_plot_order_dist(rfm_order, flip = TRUE)

# plotly
rfm_plot_order_dist(rfm_order, interactive = TRUE)

# using customer data
analysis_date <- as.Date('2007-01-01')
rfm_customer <- rfm_table_customer(rfm_data_customer, customer_id,
number_of_orders, recency_days, revenue, analysis_date)

# order distribution
rfm_plot_order_dist(rfm_customer)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rfm-plots.R
\name{rfm_plot_histogram}
\alias{rfm_plot_histogram}
\alias{rfm_histograms}
\title{RFM histograms}
\usage{
rfm_plot_histogram(
  rfm_table,
  metric = "recency",
  hist_bins = 9,
  hist_color = NULL,
  plot_title = NULL,
  xaxis_label = NULL,
  yaxis_label = NULL,
  interactive = FALSE,
  print_plot = TRUE
)
}
\arguments{
\item{rfm_table}{An object of class \code{rfm_table}.}

\item{metric}{Metric to be visualized. Defaults to \code{"recency"}. Valid
 values are:
\itemize{
\item \code{"recency"}
\item \code{"frequency"}
\item \code{"monetary"}
}}

\item{hist_bins}{Number of bins of the histograms.}

\item{hist_color}{Color of the histogram.}

\item{plot_title}{Title of the plot.}

\item{xaxis_label}{X axis label.}

\item{yaxis_label}{Y axis label.}

\item{interactive}{If \code{TRUE}, uses \code{plotly} as the visualization
engine. If \code{FALSE}, uses \code{ggplot2}.}

\item{print_plot}{logical; if \code{TRUE}, prints the plot else returns a
plot object.}
}
\value{
Histograms
}
\description{
Histograms of recency, frequency and monetary value.
}
\section{Deprecated Functions}{

\code{rfm_histograms()} has been deprecated and will be made defunct. It has
been provided for compatibility with older versions only, and will be made
defunct at the next release.

Instead use the replacement function \code{rfm_plot_histogram()}.
}

\examples{
# using transaction data
analysis_date <- as.Date('2006-12-31')
rfm_order <- rfm_table_order(rfm_data_orders, customer_id, order_date,
revenue, analysis_date)

# histogram
# ggplot2
rfm_plot_histogram(rfm_order, metric = "frequency")

# plotly
rfm_plot_histogram(rfm_order, metric = "frequency", interactive = TRUE)

# using customer data
analysis_date <- as.Date('2007-01-01')
rfm_customer <- rfm_table_customer(rfm_data_customer, customer_id,
number_of_orders, recency_days, revenue, analysis_date)

# histogram
rfm_plot_histogram(rfm_customer)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rfm-data.R
\docType{data}
\name{rfm_data_customer}
\alias{rfm_data_customer}
\title{RFM customer data}
\format{
A tibble with 39,999 rows and 5 variables:
\describe{
  \item{customer_id}{Customer id.}
  \item{total_amount}{Total amount of all orders.}
  \item{most_recent_visit}{Date of the most recent transaction.}
  \item{number_of_purchases}{Total number of transactions/orders.}
  \item{purchase_interval}{Number of days since last transaction/order.}
  #'   \item{first_name}{First name of the customer.}
  \item{last_name}{Last name of the customer.}
  \item{email}{email id of the customer.}
}
}
\usage{
rfm_data_customer
}
\description{
A dataset containing customer level data.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rfm-data.R
\docType{data}
\name{rfm_data_orders}
\alias{rfm_data_orders}
\title{RFM transaction data}
\format{
A tibble with 49.6 rows and 3 variables:
\describe{
  \item{order_date}{Order date.}
  \item{customer_id}{Customer id}
  \item{revenue}{Transaction amount.}
  \item{first_name}{First name of the customer.}
  \item{last_name}{Last name of the customer.}
  \item{email}{email id of the customer.}
}
}
\usage{
rfm_data_orders
}
\description{
A dataset containing transactions of different customers.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rfm.R
\docType{package}
\name{rfm}
\alias{rfm}
\alias{rfm-package}
\title{\code{rfm} package}
\description{
Tools for customer segmentation analysis
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/rsquaredacademy/rfm}
  \item \url{https://rfm.rsquaredacademy.com/}
  \item Report bugs at \url{https://github.com/rsquaredacademy/rfm/issues}
}

}
\author{
\strong{Maintainer}: Aravind Hebbali \email{hebbali.aravind@gmail.com} (\href{https://orcid.org/0000-0001-9220-9669}{ORCID})

}
\keyword{internal}
